@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2007  Audacious development team
+ *  Copyright (C) 2005-2008  Audacious development team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team
@@ -23,6 +23,8 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+/* #define AUD_DEBUG */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -53,6 +55,10 @@
 #include "vfs_buffer.h"
 #include "vfs_buffered_file.h"
 
+#include "libSAD.h"
+
+#define FMT_FRACBITS(a) ( (a) == FMT_FIXED32 ? __AUDACIOUS_ASSUMED_MAD_F_FRACBITS__ : 0 )
+
 G_LOCK_DEFINE_STATIC(vis_mutex);
 
 struct _VisNode {
@@ -76,6 +82,10 @@ InputPluginData ip_data = {
 };
 
 static GList *vis_list = NULL;
+static int volume_l = -1, volume_r = -1;
+static SAD_dither_t *sad_state = NULL;
+static gint sad_nch = -1;
+static AFormat sad_fmt = -1;
 
 InputPlayback *
 get_current_input_playback(void)
@@ -110,11 +120,13 @@ input_stringify_disabled_list(void)
     MOWGLI_ITER_FOREACH(node, ip_data.input_list)
     {
         Plugin *plugin = (Plugin *) node->data;
-        gchar *filename = g_path_get_basename(plugin->filename);
+        gchar *filename;
 
         if (plugin->enabled)
             continue;
 
+        filename = g_path_get_basename(plugin->filename);
+
         if (list->len > 0)
             g_string_append(list, ":");
 
@@ -141,139 +153,65 @@ free_vis_data(void)
     G_UNLOCK(vis_mutex);
 }
 
-static void
-convert_to_s16_ne(AFormat fmt, gpointer ptr, gint16 * left,
-                  gint16 * right, gint nch, gint max)
-{
-    gint16 *ptr16;
-    guint16 *ptru16;
-    guint8 *ptru8;
-    gint i;
-
-    switch (fmt) {
-    case FMT_U8:
-        ptru8 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++)
-                left[i] = ((*ptru8++) ^ 128) << 8;
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = ((*ptru8++) ^ 128) << 8;
-                right[i] = ((*ptru8++) ^ 128) << 8;
-            }
-        break;
-    case FMT_S8:
-        ptru8 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++)
-                left[i] = (*ptru8++) << 8;
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = (*ptru8++) << 8;
-                right[i] = (*ptru8++) << 8;
-            }
-        break;
-    case FMT_U16_LE:
-        ptru16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++, ptru16++)
-                left[i] = GUINT16_FROM_LE(*ptru16) ^ 32768;
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = GUINT16_FROM_LE(*ptru16) ^ 32768;
-                ptru16++;
-                right[i] = GUINT16_FROM_LE(*ptru16) ^ 32768;
-                ptru16++;
-            }
-        break;
-    case FMT_U16_BE:
-        ptru16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++, ptru16++)
-                left[i] = GUINT16_FROM_BE(*ptru16) ^ 32768;
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = GUINT16_FROM_BE(*ptru16) ^ 32768;
-                ptru16++;
-                right[i] = GUINT16_FROM_BE(*ptru16) ^ 32768;
-                ptru16++;
-            }
-        break;
-    case FMT_U16_NE:
-        ptru16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++)
-                left[i] = (*ptru16++) ^ 32768;
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = (*ptru16++) ^ 32768;
-                right[i] = (*ptru16++) ^ 32768;
-            }
-        break;
-    case FMT_S16_LE:
-        ptr16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++, ptr16++)
-                left[i] = GINT16_FROM_LE(*ptr16);
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = GINT16_FROM_LE(*ptr16);
-                ptr16++;
-                right[i] = GINT16_FROM_LE(*ptr16);
-                ptr16++;
-            }
-        break;
-    case FMT_S16_BE:
-        ptr16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++, ptr16++)
-                left[i] = GINT16_FROM_BE(*ptr16);
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = GINT16_FROM_BE(*ptr16);
-                ptr16++;
-                right[i] = GINT16_FROM_BE(*ptr16);
-                ptr16++;
-            }
-        break;
-    case FMT_S16_NE:
-        ptr16 = ptr;
-        if (nch == 1)
-            for (i = 0; i < max; i++)
-                left[i] = (*ptr16++);
-        else
-            for (i = 0; i < max; i++) {
-                left[i] = (*ptr16++);
-                right[i] = (*ptr16++);
-            }
-        break;
-    }
-}
-
 InputVisType
 input_get_vis_type()
 {
     return INPUT_VIS_OFF;
 }
 
+static SAD_dither_t*
+init_sad(AFormat fmt, gint nch)
+{
+    gint ret;
+
+    SAD_buffer_format in, out;
+    in.sample_format = sadfmt_from_afmt(fmt);
+    in.fracbits = FMT_FRACBITS(fmt);
+    in.channels = nch;
+    in.channels_order = SAD_CHORDER_INTERLEAVED;
+    in.samplerate = 0;
+    
+    out.sample_format = SAD_SAMPLE_S16;
+    out.fracbits = 0;
+    out.channels = nch;
+    out.channels_order = SAD_CHORDER_SEPARATED; /* sic! --asphyx */
+    out.samplerate = 0;
+
+    AUDDBG("fmt=%d, nch=%d\n", fmt, nch);
+    SAD_dither_t *state = SAD_dither_init(&in, &out, &ret);
+    if (state != NULL) SAD_dither_set_dither(state, FALSE);
+    return state;
+}
+
 void
 input_add_vis_pcm(gint time, AFormat fmt, gint nch, gint length, gpointer ptr)
 {
     VisNode *vis_node;
     gint max;
+    
+    if (nch > 2) return;
 
-    max = length / nch;
-    if (fmt == FMT_U16_LE || fmt == FMT_U16_BE || fmt == FMT_U16_NE ||
-        fmt == FMT_S16_LE || fmt == FMT_S16_BE || fmt == FMT_S16_NE)
-        max /= 2;
+    if (sad_state == NULL || nch != sad_nch || fmt != sad_fmt) {
+        if(sad_state != NULL) SAD_dither_free(sad_state);
+        sad_state = init_sad(fmt, nch);
+        if(sad_state == NULL) return;
+        sad_nch = nch;
+        sad_fmt = fmt;
+    }
+
+    max = length / nch / FMT_SIZEOF(fmt);
     max = CLAMP(max, 0, 512);
 
     vis_node = g_slice_new0(VisNode);
     vis_node->time = time;
     vis_node->nch = nch;
     vis_node->length = max;
-    convert_to_s16_ne(fmt, ptr, vis_node->data[0], vis_node->data[1], nch,
-                      max);
+
+    gint16 *tmp[2];
+
+    tmp[0] = vis_node->data[0];
+    tmp[1] = vis_node->data[1];
+    SAD_dither_process_buffer(sad_state, ptr, tmp, max);
 
     G_LOCK(vis_mutex);
     vis_list = g_list_append(vis_list, vis_node);
@@ -323,6 +261,7 @@ input_do_check_file(InputPlugin *ip, VFSFile *fd, gchar *filename_proxy, gboolea
          (ip->probe_for_tuple && ip->have_subtune == TRUE) ||
          (ip->probe_for_tuple && (cfg.use_pl_metadata && (!loading || (loading && cfg.get_info_on_load)))) ) {
 
+        plugin_set_current((Plugin *)ip);
         Tuple *tuple = ip->probe_for_tuple(filename_proxy, fd);
 
         if (tuple != NULL) {
@@ -336,6 +275,7 @@ input_do_check_file(InputPlugin *ip, VFSFile *fd, gchar *filename_proxy, gboolea
     }
 
     else if (ip->is_our_file_from_vfs != NULL) {
+	plugin_set_current((Plugin *)ip);
         result = ip->is_our_file_from_vfs(filename_proxy, fd);
 
         if (result > 0) {
@@ -347,6 +287,7 @@ input_do_check_file(InputPlugin *ip, VFSFile *fd, gchar *filename_proxy, gboolea
     }
 
     else if (ip->is_our_file != NULL) {
+	plugin_set_current((Plugin *)ip);
         result = ip->is_our_file(filename_proxy);
 
         if (result > 0) {
@@ -432,7 +373,10 @@ input_check_file(const gchar *filename, gboolean loading)
     /* cue:// cdda:// tone:// tact:// */
     if ((ip = uri_get_plugin(filename_proxy)) != NULL && ip->enabled) {
         if (ip->is_our_file != NULL)
+	{
+	    plugin_set_current((Plugin *)ip);
             ret = ip->is_our_file(filename_proxy);
+	}
         else
             ret = 0;
         if (ret > 0) {
@@ -557,7 +501,10 @@ input_set_eq(gint on, gfloat preamp, gfloat * bands)
         return;
 
     if (playback->plugin->set_eq)
+    {
+        plugin_set_current((Plugin *)(playback->plugin));
         playback->plugin->set_eq(on, preamp, bands);
+    }
 }
 
 
@@ -565,46 +512,52 @@ Tuple *
 input_get_song_tuple(const gchar * filename)
 {
     InputPlugin *ip = NULL;
-    Tuple *input;
+    Tuple *input = NULL;
     gchar *ext = NULL;
     gchar *filename_proxy;
     ProbeResult *pr;
 
     if (filename == NULL)
-    return NULL;
+        return NULL;
 
     filename_proxy = g_strdup(filename);
 
     pr = input_check_file(filename_proxy, FALSE);
 
-    if (!pr)
+    if (!pr) {
+        g_free(filename_proxy);
         return NULL;
+    }
 
     ip = pr->ip;
 
     g_free(pr);
 
     if (ip && ip->get_song_tuple)
+    {
+        plugin_set_current((Plugin *)ip);
         input = ip->get_song_tuple(filename_proxy);
+    }
     else
     {
-        gchar *tmp;
+        gchar *scratch;
 
-        input = tuple_new();
+        scratch = uri_to_display_basename(filename);
+        tuple_associate_string(input, FIELD_FILE_NAME, NULL, scratch);
+        g_free(scratch);
 
-        tmp = g_strdup(filename);
-        if ((ext = strrchr(tmp, '.')))
-            *ext = '\0';
+        scratch = uri_to_display_dirname(filename);
+        tuple_associate_string(input, FIELD_FILE_PATH, NULL, scratch);
+        g_free(scratch);
 
-        tuple_associate_string(input, FIELD_FILE_NAME, NULL, g_path_get_basename(tmp));
+        ext = strrchr(filename, '.');
+        if (ext != NULL) {
+            ++ext;
+            tuple_associate_string(input, FIELD_FILE_EXT, NULL, ext);
+        }
 
-        if (ext)
-            tuple_associate_string(input, FIELD_FILE_EXT, NULL, ext + 1);
-
-        tuple_associate_string(input, FIELD_FILE_PATH, NULL, g_path_get_dirname(tmp));
         tuple_associate_int(input, FIELD_LENGTH, NULL, -1);
 
-        g_free(tmp);
     }
 
     g_free(filename_proxy);
@@ -707,7 +660,10 @@ input_file_info_box(const gchar * filename)
     g_free(pr);
 
     if (ip->file_info_box)
+    {
+        plugin_set_current((Plugin *)ip);
         ip->file_info_box(filename_proxy);
+    }
     else
         input_general_file_info_box(filename, ip);
 
@@ -742,6 +698,7 @@ input_scan_dir(const gchar * path)
         if (!ip->enabled)
             continue;
 
+        plugin_set_current((Plugin *)ip);
         if ((result = ip->scan_dir(path_proxy)))
             break;
     }
@@ -754,17 +711,11 @@ input_scan_dir(const gchar * path)
 void
 input_get_volume(gint * l, gint * r)
 {
-    InputPlayback *playback;
+    if (volume_l == -1 || volume_r == -1)
+	output_get_volume(&volume_l, &volume_r);
 
-    *l = -1;
-    *r = -1;
-    if (playback_get_playing()) {
-        if ((playback = get_current_input_playback()) != NULL &&
-            playback->plugin->get_volume &&
-            playback->plugin->get_volume(l, r))
-            return;
-    }
-    output_get_volume(l, r);
+    *l = volume_l;
+    *r = volume_r;
 }
 
 void
@@ -778,13 +729,19 @@ input_set_volume(gint l, gint r)
     h_vol[1] = r;
     hook_call("volume set", h_vol);
     
-    if (playback_get_playing() &&
-        (playback = get_current_input_playback()) != NULL &&
-        playback->plugin->set_volume &&
-        playback->plugin->set_volume(l, r))
-        return;
+    if (playback_get_playing())
+        if ((playback = get_current_input_playback()) != NULL)
+	    if (playback->plugin->set_volume != NULL)
+	    {
+	        plugin_set_current((Plugin *)(playback->plugin));
+	        if (playback->plugin->set_volume(l, r))
+		    return;
+	    }
 
     output_set_volume(l, r);
+
+    volume_l = l;
+    volume_r = r;
 }
 
 void
