@@ -6,7 +6,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 2 of the License.
+ *  the Free Software Foundation; under version 3 of the License.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,8 +14,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ *
+ *  The Audacious team does not consider modular code linking to
+ *  Audacious or using our public API to be a derived work.
  */
 
 #ifndef PLAYBACK_H
@@ -24,8 +26,10 @@
 #include <glib.h>
 
 #include "playlist.h"
+#include "plugin.h"
 
 gint playback_get_time(void);
+gint playback_get_length(void);
 void playback_initiate(void);
 void playback_pause(void);
 void playback_stop(void);
@@ -34,5 +38,10 @@ gboolean playback_get_playing(void);
 gboolean playback_get_paused(void);
 void playback_seek(gint time);
 void playback_seek_relative(gint offset);
+void playback_eof(void);
+void playback_error(void);
+InputPlayback *playback_new(void);
+void playback_free(InputPlayback *);
+void playback_run(InputPlayback *);
 
 #endif
