@@ -23,6 +23,7 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+/*#define AUD_DEBUG*/
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -139,8 +140,6 @@ static GdkBitmap *skin_create_transparent_mask(const gchar *,
                                                GdkWindow *,
                                                gint, gint, gboolean);
 
-static void skin_setup_masks(Skin * skin);
-
 static void skin_set_default_vis_color(Skin * skin);
 
 void
@@ -158,20 +157,21 @@ skin_unlock(Skin * skin)
 gboolean
 bmp_active_skin_reload(void) 
 {
+    AUDDBG("\n");
     return bmp_active_skin_load(bmp_active_skin->path); 
 }
 
 gboolean
 bmp_active_skin_load(const gchar * path)
 {
+    AUDDBG("%s\n", path);
     g_return_val_if_fail(bmp_active_skin != NULL, FALSE);
 
     if (!skin_load(bmp_active_skin, path)) {
+        AUDDBG("loading failed\n");
         return FALSE;
     }
 
-    skin_setup_masks(bmp_active_skin);
-
     ui_skinned_window_draw_all(mainwin);
     ui_skinned_window_draw_all(equalizerwin);
     ui_skinned_window_draw_all(playlistwin);
@@ -190,10 +190,10 @@ void
 skin_pixmap_free(SkinPixmap * p)
 {
     g_return_if_fail(p != NULL);
-    g_return_if_fail(p->pixmap != NULL);
+    g_return_if_fail(p->pixbuf != NULL);
 
-    g_object_unref(p->pixmap);
-    p->pixmap = NULL;
+    g_object_unref(p->pixbuf);
+    p->pixbuf = NULL;
 }
 
 Skin *
@@ -205,6 +205,12 @@ skin_new(void)
     return skin;
 }
 
+/**
+ * Frees the data associated for skin.
+ *
+ * Does not free skin itself or lock variable so that the skin can immediately
+ * populated with new skin data if needed.
+ */
 void
 skin_free(Skin * skin)
 {
@@ -212,23 +218,30 @@ skin_free(Skin * skin)
 
     g_return_if_fail(skin != NULL);
 
-    skin_lock(skin);
-
     for (i = 0; i < SKIN_PIXMAP_COUNT; i++)
         skin_pixmap_free(&skin->pixmaps[i]);
 
-    for (i = 0; i < SKIN_PIXMAP_COUNT; i++) {
+    for (i = 0; i < SKIN_MASK_COUNT; i++) {
         if (skin->masks[i])
             g_object_unref(skin->masks[i]);
-        if (skin->ds_masks[i])
-            g_object_unref(skin->ds_masks[i]);
+        if (skin->scaled_masks[i])
+            g_object_unref(skin->scaled_masks[i]);
 
         skin->masks[i] = NULL;
-        skin->ds_masks[i] = NULL;
+        skin->scaled_masks[i] = NULL;
     }
 
+    for (i = 0; i < SKIN_COLOR_COUNT; i++) {
+        if (skin->colors[i])
+            g_free(skin->colors[i]);
+
+        skin->colors[i] = NULL;
+    }
+
+    g_free(skin->path);
+    skin->path = NULL;
+
     skin_set_default_vis_color(skin);
-    skin_unlock(skin);
 }
 
 void
@@ -298,60 +311,20 @@ skin_pixmap_locate(const gchar * dirname, gchar ** basenames)
     return NULL;
 }
 
-/* FIXME: this function is temporary. It will be removed when the skinning system
-   uses GdkPixbuf in place of GdkPixmap */
-
-static GdkPixmap *
-pixmap_new_from_file(const gchar * filename)
+/**
+ * Creates possible file names for a pixmap.
+ *
+ * Basically this makes list of all possible file names that pixmap data
+ * can be found from by using the static ext_targets variable to get all
+ * possible extensions that pixmap file might have.
+ */
+static gchar **
+skin_pixmap_create_basenames(const SkinPixmapIdMapping * pixmap_id_mapping)
 {
-    GdkPixbuf *pixbuf, *pixbuf2;
-    GdkPixmap *pixmap;
-    gint width, height;
-
-    if (!(pixbuf = gdk_pixbuf_new_from_file(filename, NULL)))
-        return NULL;
-
-    width = gdk_pixbuf_get_width(pixbuf);
-    height = gdk_pixbuf_get_height(pixbuf);
-
-    if (!(pixmap = gdk_pixmap_new(mainwin->window, width, height,
-                                  gdk_rgb_get_visual()->depth))) {
-        g_object_unref(pixbuf);
-        return NULL;
-    }
-
-    pixbuf2 = audacious_create_colorized_pixbuf(pixbuf, cfg.colorize_r, cfg.colorize_g, cfg.colorize_b);
-    g_object_unref(pixbuf);
-
-    GdkGC *gc;
-    gc = gdk_gc_new(pixmap);
-    gdk_draw_pixbuf(pixmap, gc, pixbuf2, 0, 0, 0, 0, width, height, GDK_RGB_DITHER_MAX, 0, 0);
-    g_object_unref(gc);
-    g_object_unref(pixbuf2);
-
-    return pixmap;
-}
-
-static gboolean
-skin_load_pixmap_id(Skin * skin, SkinPixmapId id, const gchar * path_p)
-{
-    const gchar *path;
-    gchar *filename;
-    gint width, height;
-    const SkinPixmapIdMapping *pixmap_id_mapping;
-    GdkPixmap *gpm;
-    SkinPixmap *pm = NULL;
-    gchar *basenames[EXTENSION_TARGETS * 2 + 1]; /* alternate basenames */
+    gchar **basenames = g_malloc0(sizeof(gchar*) * (EXTENSION_TARGETS * 2 + 1));
     gint i, y;
 
-    g_return_val_if_fail(skin != NULL, FALSE);
-    g_return_val_if_fail(id < SKIN_PIXMAP_COUNT, FALSE);
-
-    pixmap_id_mapping = skin_pixmap_id_lookup(id);
-    g_return_val_if_fail(pixmap_id_mapping != NULL, FALSE);
-
-    memset(&basenames, 0, sizeof(basenames));
-
+    // Create list of all possible image formats that can be loaded
     for (i = 0, y = 0; i < EXTENSION_TARGETS; i++, y++)
     {
         basenames[y] =
@@ -363,34 +336,75 @@ skin_load_pixmap_id(Skin * skin, SkinPixmapId id, const gchar * path_p)
                                 ext_targets[i]);
     }
 
-    path = path_p ? path_p : skin->path;
-    filename = skin_pixmap_locate(path, basenames);
+    return basenames;
+}
 
+/**
+ * Frees the data allocated by skin_pixmap_create_basenames
+ */
+static void
+skin_pixmap_free_basenames(gchar ** basenames)
+{
+    int i;
     for (i = 0; basenames[i] != NULL; i++)
     {
-         g_free(basenames[i]);
-         basenames[i] = NULL;
+        g_free(basenames[i]);
+        basenames[i] = NULL;
     }
+    g_free(basenames);
+}
+
+/**
+ * Locates a pixmap file for skin.
+ */
+static gchar *
+skin_pixmap_locate_basenames(const Skin * skin,
+                             const SkinPixmapIdMapping * pixmap_id_mapping,
+                             const gchar * path_p)
+{
+    gchar *filename = NULL;
+    const gchar *path = path_p ? path_p : skin->path;
+    gchar **basenames = skin_pixmap_create_basenames(pixmap_id_mapping);
+
+    filename = skin_pixmap_locate(path, basenames);
+
+    skin_pixmap_free_basenames(basenames);
+
+    return filename;
+}
+
+
+static gboolean
+skin_load_pixmap_id(Skin * skin, SkinPixmapId id, const gchar * path_p)
+{
+    const SkinPixmapIdMapping *pixmap_id_mapping;
+    gchar *filename;
+    SkinPixmap *pm = NULL;
+
+    g_return_val_if_fail(skin != NULL, FALSE);
+    g_return_val_if_fail(id < SKIN_PIXMAP_COUNT, FALSE);
+
+    pixmap_id_mapping = skin_pixmap_id_lookup(id);
+    g_return_val_if_fail(pixmap_id_mapping != NULL, FALSE);
+
+    filename = skin_pixmap_locate_basenames(skin, pixmap_id_mapping, path_p);
 
     if (filename == NULL)
         return FALSE;
 
-    if (!(gpm = pixmap_new_from_file(filename))) {
-        g_warning("loading of %s failed", filename);
-        g_free(filename);
-        return FALSE;
-    }
+    AUDDBG("loaded %s\n", filename);
+
+    pm = &skin->pixmaps[id];
+    GdkPixbuf *pix = gdk_pixbuf_new_from_file(filename, NULL);
+    pm->pixbuf = audacious_create_colorized_pixbuf(pix, cfg.colorize_r, cfg.colorize_g, cfg.colorize_b);
+    g_object_unref(pix);
+    pm->width = gdk_pixbuf_get_width(pm->pixbuf);
+    pm->height = gdk_pixbuf_get_height(pm->pixbuf);
+    pm->current_width = pm->width;
+    pm->current_height = pm->height;
 
     g_free(filename);
 
-    gdk_drawable_get_size(GDK_DRAWABLE(gpm), &width, &height);
-    pm = &skin->pixmaps[id];
-    pm->pixmap = gpm;
-    pm->width = width;
-    pm->height = height;
-    pm->current_width = width;
-    pm->current_height = height;
-
     return TRUE;
 }
 
@@ -406,30 +420,13 @@ skin_mask_create(Skin * skin,
                                      skin_mask_info[id].width,
                                      skin_mask_info[id].height, FALSE);
 
-    skin->ds_masks[id] =
+    skin->scaled_masks[id] =
         skin_create_transparent_mask(path, "region.txt",
                                      skin_mask_info[id].inistr, window,
                                      skin_mask_info[id].width * 2,
                                      skin_mask_info[id].height * 2, TRUE);
 }
 
-static void
-skin_setup_masks(Skin * skin)
-{
-    GdkBitmap *mask;
-
-    if (cfg.show_wm_decorations)
-        return;
-
-    if (cfg.player_visible) {
-        mask = skin_get_mask(skin, SKIN_MASK_MAIN + cfg.player_shaded);
-        gtk_widget_shape_combine_mask(mainwin, mask, 0, 0);
-    }
-
-    mask = skin_get_mask(skin, SKIN_MASK_EQ + cfg.equalizer_shaded);
-    gtk_widget_shape_combine_mask(equalizerwin, mask, 0, 0);
-}
-
 static GdkBitmap *
 create_default_mask(GdkWindow * parent, gint w, gint h)
 {
@@ -470,7 +467,7 @@ skin_calc_luminance(GdkColor * c)
 }
 
 static void
-skin_get_textcolors(GdkPixmap * text, GdkColor * bgc, GdkColor * fgc)
+skin_get_textcolors(GdkPixbuf * pix, GdkColor * bgc, GdkColor * fgc)
 {
     /*
      * Try to extract reasonable background and foreground colors
@@ -481,9 +478,12 @@ skin_get_textcolors(GdkPixmap * text, GdkColor * bgc, GdkColor * fgc)
     GdkColormap *cm;
     gint i;
 
-    g_return_if_fail(text != NULL);
+    g_return_if_fail(pix != NULL);
     g_return_if_fail(GDK_IS_WINDOW(playlistwin->window));
 
+    GdkPixmap *text = gdk_pixmap_new(NULL, gdk_pixbuf_get_width(pix), gdk_pixbuf_get_height(pix), gdk_rgb_get_visual()->depth);
+    gdk_draw_pixbuf(text, NULL, pix, 0, 0, 0, 0, gdk_pixbuf_get_width(pix), gdk_pixbuf_get_height(pix),
+                    GDK_RGB_DITHER_NONE, 0, 0);
     /* Get the first line of text */
     gi = gdk_drawable_get_image(text, 0, 0, 152, 6);
     cm = gdk_drawable_get_colormap(playlistwin->window);
@@ -510,6 +510,7 @@ skin_get_textcolors(GdkPixmap * text, GdkColor * bgc, GdkColor * fgc)
         }
     }
     g_object_unref(gi);
+    g_object_unref(text);
 }
 
 gboolean
@@ -529,13 +530,13 @@ init_skins(const gchar * path)
 
     if (!bmp_active_skin_load(path)) {
         if (path != NULL)
-            g_message("Unable to load skin (%s), trying default...", path);
+            AUDDBG("Unable to load skin (%s), trying default...\n", path);
         else
-            g_message("Skin not defined: trying default...");
+            AUDDBG("Skin not defined: trying default...\n");
 
         /* can't load configured skin, retry with default */
         if (!bmp_active_skin_load(BMP_DEFAULT_SKIN_PATH)) {
-            g_message("Unable to load default skin (%s)! Giving up.",
+            AUDDBG("Unable to load default skin (%s)! Giving up.\n",
                       BMP_DEFAULT_SKIN_PATH);
             return FALSE;
         }
@@ -547,6 +548,12 @@ init_skins(const gchar * path)
     return TRUE;
 }
 
+void cleanup_skins()
+{
+    skin_destroy(bmp_active_skin);
+}
+
+
 /*
  * Opens and parses a skin's hints file.
  * Hints files are somewhat like "scripts" in Winamp3/5.
@@ -636,6 +643,8 @@ skin_parse_hints(Skin * skin, gchar *path_p)
         return;
 
     inifile = open_ini_file(filename);
+    if (!inifile)
+        return;
 
     tmp = read_ini_string(inifile, "skin", "mainwinOthertext");
 
@@ -1185,7 +1194,7 @@ skin_load_color(INIFile *inifile,
 
     if (inifile || default_hex) {
         if (inifile) {
-            value = g_strdup(read_ini_string(inifile, section, key));
+            value = read_ini_string(inifile, section, key);
             if (value == NULL) {
                 value = g_strdup(default_hex);
             }
@@ -1233,7 +1242,7 @@ skin_create_transparent_mask(const gchar * path,
                              const gchar * section,
                              GdkWindow * window,
                              gint width,
-                             gint height, gboolean doublesize)
+                             gint height, gboolean scale)
 {
     GdkBitmap *mask = NULL;
     GdkGC *gc = NULL;
@@ -1287,10 +1296,10 @@ skin_create_transparent_mask(const gchar * path,
             gpoints = g_new(GdkPoint, g_array_index(num, gint, i));
             for (k = 0; k < g_array_index(num, gint, i); k++) {
                 gpoints[k].x =
-                    g_array_index(point, gint, j + k * 2) * (1 + doublesize);
+                    g_array_index(point, gint, j + k * 2) * (scale ? cfg.scale_factor : 1 );
                 gpoints[k].y =
                     g_array_index(point, gint,
-                                  j + k * 2 + 1) * (1 + doublesize);
+                                  j + k * 2 + 1) * (scale ? cfg.scale_factor : 1);
             }
             j += k * 2;
             gdk_draw_polygon(mask, gc, TRUE, gpoints,
@@ -1354,29 +1363,25 @@ skin_load_viscolor(Skin * skin, const gchar * path, const gchar * basename)
 static void
 skin_numbers_generate_dash(Skin * skin)
 {
-    GdkGC *gc;
-    GdkPixmap *pixmap;
+    GdkPixbuf *pixbuf;
     SkinPixmap *numbers;
 
     g_return_if_fail(skin != NULL);
 
     numbers = &skin->pixmaps[SKIN_NUMBERS];
-    if (!numbers->pixmap || numbers->current_width < 99)
+    if (!numbers->pixbuf || numbers->current_width < 99)
         return;
 
-    pixmap = gdk_pixmap_new(NULL, 108,
-                            numbers->current_height,
-                            gdk_rgb_get_visual()->depth);
-    gc = gdk_gc_new(pixmap);
+    pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
+                            108, numbers->current_height);
 
-    skin_draw_pixmap(NULL, skin, pixmap, gc, SKIN_NUMBERS, 0, 0, 0, 0, 99, numbers->current_height);
-    skin_draw_pixmap(NULL, skin, pixmap, gc, SKIN_NUMBERS, 90, 0, 99, 0, 9, numbers->current_height);
-    skin_draw_pixmap(NULL, skin, pixmap, gc, SKIN_NUMBERS, 20, 6, 101, 6, 5, 1);
+    skin_draw_pixbuf(NULL, skin, pixbuf, SKIN_NUMBERS, 0, 0, 0, 0, 99, numbers->current_height);
+    skin_draw_pixbuf(NULL, skin, pixbuf, SKIN_NUMBERS, 90, 0, 99, 0, 9, numbers->current_height);
+    skin_draw_pixbuf(NULL, skin, pixbuf, SKIN_NUMBERS, 20, 6, 101, 6, 5, 1);
 
-    g_object_unref(numbers->pixmap);
-    g_object_unref(gc);
+    g_object_unref(numbers->pixbuf);
 
-    numbers->pixmap = pixmap;
+    numbers->pixbuf = pixbuf;
     numbers->current_width = 108;
     numbers->width = 108;
 }
@@ -1417,29 +1422,32 @@ skin_load_cursor(Skin * skin, const gchar * dirname)
 static gboolean
 skin_load_pixmaps(Skin * skin, const gchar * path)
 {
-    GdkPixmap *text_pm;
+    GdkPixbuf *text_pb;
     guint i;
     gchar *filename;
     INIFile *inifile;
+    
+    if(!skin) return FALSE;
+    if(!path) return FALSE;
+
+    AUDDBG("Loading pixmaps in %s\n", path);
 
     for (i = 0; i < SKIN_PIXMAP_COUNT; i++)
-        skin_load_pixmap_id(skin, i, path);
+        if (!skin_load_pixmap_id(skin, i, path))
+            return FALSE;
 
-    text_pm = skin->pixmaps[SKIN_TEXT].pixmap;
+    text_pb = skin->pixmaps[SKIN_TEXT].pixbuf;
 
-    if (text_pm)
-        skin_get_textcolors(text_pm, skin->textbg, skin->textfg);
+    if (text_pb)
+        skin_get_textcolors(text_pb, skin->textbg, skin->textfg);
 
-    if (skin->pixmaps[SKIN_NUMBERS].pixmap &&
+    if (skin->pixmaps[SKIN_NUMBERS].pixbuf &&
         skin->pixmaps[SKIN_NUMBERS].width < 108 )
         skin_numbers_generate_dash(skin);
 
     filename = find_file_recursively(path, "pledit.txt");
     inifile = open_ini_file(filename);
 
-    if (!inifile)
-        return FALSE;
-
     skin->colors[SKIN_PLEDIT_NORMAL] =
         skin_load_color(inifile, "Text", "Normal", "#2499ff");
     skin->colors[SKIN_PLEDIT_CURRENT] =
@@ -1449,9 +1457,11 @@ skin_load_pixmaps(Skin * skin, const gchar * path)
     skin->colors[SKIN_PLEDIT_SELECTEDBG] =
         skin_load_color(inifile, "Text", "SelectedBG", "#0a124a");
 
+    if (inifile)
+        close_ini_file(inifile);
+
     if (filename)
         g_free(filename);
-    close_ini_file(inifile);
 
     skin_mask_create(skin, path, SKIN_MASK_MAIN, mainwin->window);
     skin_mask_create(skin, path, SKIN_MASK_MAIN_SHADE, mainwin->window);
@@ -1485,11 +1495,34 @@ skin_set_gtk_theme(GtkSettings * settings, Skin * skin)
     g_free(tmp);
 }
 
+/**
+ * Checks if all pixmap files exist that skin needs.
+ */
+static gboolean
+skin_check_pixmaps(const Skin * skin, const gchar * skin_path)
+{
+    guint i;
+    for (i = 0; i < SKIN_PIXMAP_COUNT; i++)
+    {
+        gchar *filename = skin_pixmap_locate_basenames(skin,
+                                                       skin_pixmap_id_lookup(i),
+                                                       skin_path);
+        if (!filename)
+            return FALSE;
+        g_free(filename);
+    }
+    return TRUE;
+}
+
 static gboolean
 skin_load_nolock(Skin * skin, const gchar * path, gboolean force)
 {
     GtkSettings *settings;
-    gchar *cpath, *gtkrcpath;
+    gchar *gtkrcpath;
+    gchar *newpath, *skin_path;
+    int archive = 0;
+
+    AUDDBG("Attempt to load skin \"%s\"\n", path);
 
     g_return_val_if_fail(skin != NULL, FALSE);
     g_return_val_if_fail(path != NULL, FALSE);
@@ -1498,68 +1531,65 @@ skin_load_nolock(Skin * skin, const gchar * path, gboolean force)
     if (!g_file_test(path, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_DIR))
         return FALSE;
    
+    if(force) AUDDBG("reloading forced!\n");
     if (!force && skin->path && !strcmp(skin->path, path)) {
+        AUDDBG("skin %s already loaded\n", path);
         return FALSE;
     }
     
+    if (file_is_archive(path)) {
+        AUDDBG("Attempt to load archive\n");
+        if (!(skin_path = archive_decompress(path))) {
+            AUDDBG("Unable to extract skin archive (%s)\n", path);
+            return FALSE;
+        }
+        archive = 1;
+    } else {
+        skin_path = g_strdup(path);
+    }
+
+    // Check if skin path has all necessary files.
+    if (!skin_check_pixmaps(skin, skin_path)) {
+        if(archive) del_directory(skin_path);
+        g_free(skin_path);
+        AUDDBG("Skin path (%s) doesn't have all wanted pixmaps\n", skin_path);
+        return FALSE;
+    }
+
+    // skin_free() frees skin->path and variable path can actually be skin->path
+    // and we want to get the path before possibly freeing it.
+    newpath = g_strdup(path);
+    skin_free(skin);
+    skin->path = newpath;
+
     memset(&(skin->properties), 0, sizeof(SkinProperties)); /* do it only if all tests above passed! --asphyx */
-      
+    
     skin_current_num++;
 
-    skin->path = g_strdup(path);
+    /* Parse the hints for this skin. */
+    skin_parse_hints(skin, skin_path);
 
+    if (!skin_load_pixmaps(skin, skin_path)) {
+        if(archive) del_directory(skin_path);
+        g_free(skin_path);
+        AUDDBG("Skin loading failed\n");
+        return FALSE;
+    }
 
+    skin_load_cursor(skin, skin_path);
+
+    /* restore gtk theme if changed by previous skin */
     settings = gtk_settings_get_default();
-    
-    if (original_gtk_theme != NULL)
-    {
+
+    if (original_gtk_theme != NULL) {
         gtk_settings_set_string_property(settings, "gtk-theme-name",
-                                         original_gtk_theme, "audacious");
+                                              original_gtk_theme, "audacious");
         g_free(original_gtk_theme);
         original_gtk_theme = NULL;
     }
 
-
-    if (!file_is_archive(path)) {
-        /* Parse the hints for this skin. */
-        skin_parse_hints(skin, NULL);
-
-        if (!skin_load_pixmaps(skin, path))
-            return FALSE;
-
-        skin_load_cursor(skin, path);
-
 #ifndef _WIN32
-        if (!cfg.disable_inline_gtk) {
-            gtkrcpath = find_path_recursively(skin->path, "gtkrc");
-            if (gtkrcpath != NULL)
-                skin_set_gtk_theme(settings, skin);
-            g_free(gtkrcpath);
-        }
-#endif
-
-        return TRUE;
-    }
-    
-    if (!(cpath = archive_decompress(path))) {
-        g_message("Unable to extract skin archive (%s)", path);
-        return FALSE;
-    }
-
-    /* Parse the hints for this skin. */
-    skin_parse_hints(skin, cpath);
-
-    if (!skin_load_pixmaps(skin, cpath))
-    {
-        del_directory(cpath);
-        g_free(cpath);
-        return FALSE;
-    }
-
-    skin_load_cursor(skin, cpath);
-
-#ifndef _WIN32
-    if (!cfg.disable_inline_gtk) {
+    if (!cfg.disable_inline_gtk && !archive) {
         gtkrcpath = find_path_recursively(skin->path, "gtkrc");
         if (gtkrcpath != NULL)
             skin_set_gtk_theme(settings, skin);
@@ -1567,8 +1597,11 @@ skin_load_nolock(Skin * skin, const gchar * path, gboolean force)
     }
 #endif
 
-    del_directory(cpath);
-    g_free(cpath);
+    if(archive) del_directory(skin_path);
+    g_free(skin_path);
+
+    gtk_widget_shape_combine_mask(mainwin, skin_get_mask(bmp_active_skin, SKIN_MASK_MAIN + cfg.player_shaded), 0, 0);
+    gtk_widget_shape_combine_mask(equalizerwin, skin_get_mask(bmp_active_skin, SKIN_MASK_EQ + cfg.equalizer_shaded), 0, 0);
 
     return TRUE;
 }
@@ -1583,7 +1616,7 @@ skin_install_skin(const gchar * path)
     command = g_strdup_printf("cp %s %s",
                               path, bmp_paths[BMP_PATH_USER_SKIN_DIR]);
     if (system(command)) {
-        g_message("Unable to install skin (%s) into user directory (%s)",
+        AUDDBG("Unable to install skin (%s) into user directory (%s)\n",
                   path, bmp_paths[BMP_PATH_USER_SKIN_DIR]);
     }
     g_free(command);
@@ -1613,6 +1646,7 @@ skin_load(Skin * skin, const gchar * path)
     skin_unlock(skin);
 
     if(!ret) {
+        AUDDBG("loading failed\n");
         return FALSE; /* don't try to update anything if loading failed --asphyx */
     }
 
@@ -1645,6 +1679,7 @@ gboolean
 skin_reload_forced(void) 
 {
    gboolean error;
+   AUDDBG("\n");
 
    skin_lock(bmp_active_skin);
    error = skin_load_nolock(bmp_active_skin, bmp_active_skin->path, TRUE);
@@ -1656,6 +1691,7 @@ skin_reload_forced(void)
 void
 skin_reload(Skin * skin)
 {
+    AUDDBG("\n");
     g_return_if_fail(skin != NULL);
     skin_load_nolock(skin, skin->path, TRUE);
 }
@@ -1668,7 +1704,7 @@ skin_get_mask(Skin * skin, SkinMaskId mi)
     g_return_val_if_fail(skin != NULL, NULL);
     g_return_val_if_fail(mi < SKIN_PIXMAP_COUNT, NULL);
 
-    masks = cfg.doublesize ? skin->ds_masks : skin->masks;
+    masks = cfg.scaled ? skin->scaled_masks : skin->masks;
     return masks[mi];
 }
 
@@ -1681,13 +1717,13 @@ skin_get_color(Skin * skin, SkinColorId color_id)
 
     switch (color_id) {
     case SKIN_TEXTBG:
-        if (skin->pixmaps[SKIN_TEXT].pixmap)
+        if (skin->pixmaps[SKIN_TEXT].pixbuf)
             ret = skin->textbg;
         else
             ret = skin->def_textbg;
         break;
     case SKIN_TEXTFG:
-        if (skin->pixmaps[SKIN_TEXT].pixmap)
+        if (skin->pixmaps[SKIN_TEXT].pixbuf)
             ret = skin->textfg;
         else
             ret = skin->def_textfg;
@@ -1721,7 +1757,7 @@ skin_get_id(void)
 }
 
 void
-skin_draw_pixmap(GtkWidget *widget, Skin * skin, GdkDrawable * drawable, GdkGC * gc,
+skin_draw_pixbuf(GtkWidget *widget, Skin * skin, GdkPixbuf * pix,
                  SkinPixmapId pixmap_id,
                  gint xsrc, gint ysrc, gint xdest, gint ydest,
                  gint width, gint height)
@@ -1732,7 +1768,7 @@ skin_draw_pixmap(GtkWidget *widget, Skin * skin, GdkDrawable * drawable, GdkGC *
 
     pixmap = skin_get_pixmap(skin, pixmap_id);
     g_return_if_fail(pixmap != NULL);
-    g_return_if_fail(pixmap->pixmap != NULL);
+    g_return_if_fail(pixmap->pixbuf != NULL);
 
     /* perhaps we should use transparency or resize widget? */
     if (xsrc+width > pixmap->width || ysrc+height > pixmap->height) {
@@ -1765,8 +1801,8 @@ skin_draw_pixmap(GtkWidget *widget, Skin * skin, GdkDrawable * drawable, GdkGC *
                             return;
                     }
                     /* let's copy what's under widget */
-                    gdk_draw_drawable(drawable, gc, skin_get_pixmap(bmp_active_skin, SKIN_MAIN)->pixmap,
-                                      x, y, xdest, ydest, width, height);
+                    gdk_pixbuf_copy_area(skin_get_pixmap(bmp_active_skin, SKIN_MAIN)->pixbuf,
+                                         x, y, width, height, pix, xdest, ydest);
 
                     /* XMMS skins seems to have SKIN_MONOSTEREO with size 58x20 instead of 58x24 */
                     if (pixmap_id == SKIN_MONOSTEREO)
@@ -1785,44 +1821,46 @@ skin_draw_pixmap(GtkWidget *widget, Skin * skin, GdkDrawable * drawable, GdkGC *
 
     width = MIN(width, pixmap->width - xsrc);
     height = MIN(height, pixmap->height - ysrc);
-    gdk_draw_drawable(drawable, gc, pixmap->pixmap, xsrc, ysrc,
-                      xdest, ydest, width, height);
+    gdk_pixbuf_copy_area(pixmap->pixbuf, xsrc, ysrc, width, height,
+                         pix, xdest, ydest);
 }
 
 void
 skin_get_eq_spline_colors(Skin * skin, guint32 colors[19])
 {
     gint i;
-    GdkPixmap *pixmap;
-    GdkImage *img;
+    GdkPixbuf *pixbuf;
     SkinPixmap *eqmainpm;
-
+    guchar* pixels,*p;
+    guint rowstride, n_channels;
     g_return_if_fail(skin != NULL);
 
     eqmainpm = &skin->pixmaps[SKIN_EQMAIN];
-    if (eqmainpm->pixmap &&
-        eqmainpm->current_width >= 116 && eqmainpm->current_height >= 313)
-        pixmap = eqmainpm->pixmap;
+    if (eqmainpm->pixbuf &&
+            eqmainpm->current_width >= 116 && eqmainpm->current_height >= 313)
+        pixbuf = eqmainpm->pixbuf;
     else
         return;
 
-    if (!GDK_IS_DRAWABLE(pixmap))
-        return;
-
-    if (!(img = gdk_drawable_get_image(pixmap, 115, 294, 1, 19)))
+    if (!GDK_IS_PIXBUF(pixbuf))
         return;
 
+    pixels = gdk_pixbuf_get_pixels (pixbuf);
+    rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+    n_channels = gdk_pixbuf_get_n_channels (pixbuf);
     for (i = 0; i < 19; i++)
-        colors[i] = gdk_image_get_pixel(img, 0, i);
-
-    g_object_unref(img);
+    {
+        p = pixels + rowstride * (i + 294) + 115 * n_channels;
+        colors[i] = (p[0] << 16) | (p[1] << 8) | p[2]; 
+        /* should we really treat the Alpha channel? */
+        /*if (n_channels == 4)
+            colors[i] = (colors[i] << 8) | p[3];*/
+    }
 }
 
 
 static void
-skin_draw_playlistwin_frame_top(Skin * skin,
-                                GdkDrawable * drawable,
-                                GdkGC * gc,
+skin_draw_playlistwin_frame_top(Skin * skin, GdkPixbuf * pix,
                                 gint width, gint height, gboolean focus)
 {
     /* The title bar skin consists of 2 sets of 4 images, 1 set
@@ -1846,14 +1884,14 @@ skin_draw_playlistwin_frame_top(Skin * skin,
         y = 21;
 
     /* left corner */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 0, y, 0, 0, 25, 20);
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 0, y, 0, 0, 25, 20);
 
     /* titlebar title */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 26, y,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 26, y,
                      (width - 100) / 2, 0, 100, 20);
 
     /* titlebar right corner  */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 153, y,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 153, y,
                      width - 25, 0, 25, 20);
 
     /* tile draw the remaining frame */
@@ -1863,28 +1901,26 @@ skin_draw_playlistwin_frame_top(Skin * skin,
 
     for (i = 0; i < c / 2; i++) {
         /* left of title */
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 127, y,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 127, y,
                          25 + i * 25, 0, 25, 20);
 
         /* right of title */
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 127, y,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 127, y,
                          (width + 100) / 2 + i * 25, 0, 25, 20);
     }
 
     if (c & 1) {
         /* Odd tile count, so one remaining to draw. Here we split
          * it into two and draw half on either side of the title */
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 127, y,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 127, y,
                          ((c / 2) * 25) + 25, 0, 12, 20);
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 127, y,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 127, y,
                          (width / 2) + ((c / 2) * 25) + 50, 0, 13, 20);
     }
 }
 
 static void
-skin_draw_playlistwin_frame_bottom(Skin * skin,
-                                   GdkDrawable * drawable,
-                                   GdkGC * gc,
+skin_draw_playlistwin_frame_bottom(Skin * skin, GdkPixbuf * pix,
                                    gint width, gint height, gboolean focus)
 {
     /* The bottom frame skin consists of 1 set of 4 images. The 4
@@ -1901,7 +1937,7 @@ skin_draw_playlistwin_frame_bottom(Skin * skin,
     gint i, c;
 
     /* bottom left corner (menu buttons) */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 0, 72,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 0, 72,
                      0, height - 38, 125, 38);
 
     c = (width - 275) / 25;
@@ -1909,24 +1945,22 @@ skin_draw_playlistwin_frame_bottom(Skin * skin,
     /* draw visualization window, if width allows */
     if (c >= 3) {
         c -= 3;
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 205, 0,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 205, 0,
                          width - (150 + 75), height - 38, 75, 38);
     }
 
     /* Bottom right corner (playbuttons etc) */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT,
                      126, 72, width - 150, height - 38, 150, 38);
 
     /* Tile draw the remaining undrawn portions */
     for (i = 0; i < c; i++)
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 179, 0,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 179, 0,
                          125 + i * 25, height - 38, 25, 38);
 }
 
 static void
-skin_draw_playlistwin_frame_sides(Skin * skin,
-                                  GdkDrawable * drawable,
-                                  GdkGC * gc,
+skin_draw_playlistwin_frame_sides(Skin * skin, GdkPixbuf * pix,
                                   gint width, gint height, gboolean focus)
 {
     /* The side frames consist of 2 tile images. 1 for the left, 1 for
@@ -1940,32 +1974,28 @@ skin_draw_playlistwin_frame_sides(Skin * skin,
     /* frame sides */
     for (i = 0; i < (height - (20 + 38)) / 29; i++) {
         /* left */
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 0, 42,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 0, 42,
                          0, 20 + i * 29, 12, 29);
 
         /* right */
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 32, 42,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 32, 42,
                          width - 19, 20 + i * 29, 19, 29);
     }
 }
 
 
 void
-skin_draw_playlistwin_frame(Skin * skin,
-                            GdkDrawable * drawable, GdkGC * gc,
+skin_draw_playlistwin_frame(Skin * skin, GdkPixbuf * pix,
                             gint width, gint height, gboolean focus)
 {
-    skin_draw_playlistwin_frame_top(skin, drawable, gc, width, height, focus);
-    skin_draw_playlistwin_frame_bottom(skin, drawable, gc, width, height,
-                                       focus);
-    skin_draw_playlistwin_frame_sides(skin, drawable, gc, width, height,
-                                      focus);
+    skin_draw_playlistwin_frame_top(skin, pix, width, height, focus);
+    skin_draw_playlistwin_frame_bottom(skin, pix, width, height, focus);
+    skin_draw_playlistwin_frame_sides(skin, pix, width, height, focus);
 }
 
 
 void
-skin_draw_playlistwin_shaded(Skin * skin,
-                             GdkDrawable * drawable, GdkGC * gc,
+skin_draw_playlistwin_shaded(Skin * skin, GdkPixbuf * pix,
                              gint width, gboolean focus)
 {
     /* The shade mode titlebar skin consists of 4 images:
@@ -1978,22 +2008,21 @@ skin_draw_playlistwin_shaded(Skin * skin,
     gint i;
 
     /* left corner */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 72, 42, 0, 0, 25, 14);
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 72, 42, 0, 0, 25, 14);
 
     /* bar tile */
     for (i = 0; i < (width - 75) / 25; i++)
-        skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 72, 57,
+        skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 72, 57,
                          (i * 25) + 25, 0, 25, 14);
 
     /* right corner */
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_PLEDIT, 99, focus ? 42 : 57,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_PLEDIT, 99, focus ? 42 : 57,
                      width - 50, 0, 50, 14);
 }
 
 
 void
-skin_draw_mainwin_titlebar(Skin * skin,
-                           GdkDrawable * drawable, GdkGC * gc,
+skin_draw_mainwin_titlebar(Skin * skin, GdkPixbuf * pix,
                            gboolean shaded, gboolean focus)
 {
     /* The titlebar skin consists of 2 sets of 2 images, one for for
@@ -2022,7 +2051,7 @@ skin_draw_mainwin_titlebar(Skin * skin,
             y_offset = 15;
     }
 
-    skin_draw_pixmap(NULL, skin, drawable, gc, SKIN_TITLEBAR, 27, y_offset,
+    skin_draw_pixbuf(NULL, skin, pix, SKIN_TITLEBAR, 27, y_offset,
                      0, 0, bmp_active_skin->properties.mainwin_width, MAINWIN_TITLEBAR_HEIGHT);
 }
 
