@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; under version 2 of the License.
+ * the Free Software Foundation; under version 3 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
+ *
+ * The Audacious team does not consider modular code linking to
+ * Audacious or using our public API to be a derived work.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,7 +32,6 @@
 #include "playlist.h"
 #include "playback.h"
 #include "strings.h"
-#include "titlestring.h"
 #include "ui_fileinfopopup.h"
 #include "ui_fileinfo.h"
 
@@ -87,12 +87,12 @@ filepopup_entry_set_image(GtkWidget *filepopup_win, const gchar *entry_name,
 }
 
 static gboolean
-audacious_fileinfopopup_progress_cb(gpointer filepopup_win)
+fileinfopopup_progress_cb(gpointer filepopup_win)
 {
     GtkWidget *progressbar =
         g_object_get_data(G_OBJECT(filepopup_win), "progressbar");
     gchar *tooltip_file = g_object_get_data(G_OBJECT(filepopup_win), "file");
-    gchar *current_file;
+    gchar *current_file_u, *current_file;
     Playlist *pl;
     gint length =
         GPOINTER_TO_INT(g_object_get_data(G_OBJECT(filepopup_win), "length"));
@@ -105,15 +105,16 @@ audacious_fileinfopopup_progress_cb(gpointer filepopup_win)
 
     pos = playlist_get_position(pl);
 
-    current_file = playlist_get_filename(pl , pos);
-    time = playback_get_time();
+    current_file_u = playlist_get_filename(pl , pos);
+    g_return_val_if_fail(current_file_u != NULL, FALSE);
+    current_file = g_filename_from_uri(current_file_u, NULL, NULL);
+    g_free(current_file_u);
 
-    g_return_val_if_fail(current_file != NULL, FALSE);
-
-    if (time != -1 && length != -1 &&
+    if (playback_get_playing() && length != -1 &&
         current_file != NULL && tooltip_file != NULL &&
         !strcmp(tooltip_file, current_file) && cfg.filepopup_showprogressbar)
     {
+        time = playback_get_time();
         gchar *progress_time =
             g_strdup_printf("%d:%02d", time / 60000, (time / 1000) % 60);
         gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progressbar),
@@ -133,11 +134,12 @@ audacious_fileinfopopup_progress_cb(gpointer filepopup_win)
             gtk_widget_hide(progressbar);
     }
 
+    g_free( current_file );
     return TRUE;
 }
 
 static gboolean
-audacious_fileinfopopup_progress_check_active(GtkWidget *filepopup_win)
+fileinfopopup_progress_check_active(GtkWidget *filepopup_win)
 {
     if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(filepopup_win),"progress_sid")) == 0)
         return FALSE;
@@ -145,23 +147,23 @@ audacious_fileinfopopup_progress_check_active(GtkWidget *filepopup_win)
 }
 
 static void
-audacious_fileinfopopup_progress_init(GtkWidget *filepopup_win)
+fileinfopopup_progress_init(GtkWidget *filepopup_win)
 {
     g_object_set_data( G_OBJECT(filepopup_win) , "progress_sid" , GINT_TO_POINTER(0) );
 }
 
 static void
-audacious_fileinfopopup_progress_start(GtkWidget *filepopup_win)
+fileinfopopup_progress_start(GtkWidget *filepopup_win)
 {
     gint sid =
-        g_timeout_add(500, (GSourceFunc)audacious_fileinfopopup_progress_cb,
+        g_timeout_add(500, (GSourceFunc)fileinfopopup_progress_cb,
                       filepopup_win);
     g_object_set_data(G_OBJECT(filepopup_win), "progress_sid",
                       GINT_TO_POINTER(sid));
 }
 
 static void
-audacious_fileinfopopup_progress_stop(GtkWidget *filepopup_win)
+fileinfopopup_progress_stop(GtkWidget *filepopup_win)
 {
     gint sid = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(filepopup_win),
                                                  "progress_sid"));
@@ -173,7 +175,7 @@ audacious_fileinfopopup_progress_stop(GtkWidget *filepopup_win)
 }
 
 static void
-audacious_fileinfopopup_add_category(GtkWidget *filepopup_win,
+fileinfopopup_add_category(GtkWidget *filepopup_win,
                                      GtkWidget *filepopup_data_table,
                                      const gchar *category,
                                      const gchar *header_data,
@@ -210,7 +212,7 @@ audacious_fileinfopopup_add_category(GtkWidget *filepopup_win,
 
 
 GtkWidget *
-audacious_fileinfopopup_create(void)
+fileinfopopup_create(void)
 {
     GtkWidget *filepopup_win;
     GtkWidget *filepopup_hbox;
@@ -219,6 +221,8 @@ audacious_fileinfopopup_create(void)
     GtkWidget *filepopup_progress;
 
     filepopup_win = gtk_window_new(GTK_WINDOW_POPUP);
+    gtk_window_set_type_hint(GTK_WINDOW(filepopup_win),
+		    		 GDK_WINDOW_TYPE_HINT_TOOLTIP);
     gtk_window_set_decorated(GTK_WINDOW(filepopup_win), FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(filepopup_win), 6);
     gtk_window_set_transient_for(GTK_WINDOW(filepopup_win),
@@ -247,26 +251,26 @@ audacious_fileinfopopup_create(void)
     gtk_box_pack_start(GTK_BOX(filepopup_hbox), filepopup_data_table,
                        TRUE, TRUE, 0);
 
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Title"),
                                          "header_title", "label_title", 0);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Artist"),
                                          "header_artist", "label_artist", 1);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Album"),
                                          "header_album", "label_album", 2);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Genre"),
                                          "header_genre", "label_genre", 3);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Year"),
                                          "header_year", "label_year", 4);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Track Number"),
                                          "header_tracknum", "label_tracknum",
                                          5);
-    audacious_fileinfopopup_add_category(filepopup_win, filepopup_data_table,
+    fileinfopopup_add_category(filepopup_win, filepopup_data_table,
                                          _("Track Length"),
                                          "header_tracklen", "label_tracklen",
                                          6);
@@ -283,7 +287,7 @@ audacious_fileinfopopup_create(void)
     g_object_set_data(G_OBJECT(filepopup_win), "file", NULL);
     g_object_set_data(G_OBJECT(filepopup_win), "progressbar",
                       filepopup_progress);
-    audacious_fileinfopopup_progress_init(filepopup_win);
+    fileinfopopup_progress_init(filepopup_win);
     
     /* this will realize all widgets contained in filepopup_hbox */
     gtk_widget_show_all(filepopup_hbox);
@@ -295,11 +299,12 @@ audacious_fileinfopopup_create(void)
 }
 
 void
-audacious_fileinfopopup_destroy(GtkWidget *filepopup_win)
+fileinfopopup_destroy(GtkWidget *filepopup_win)
 {
-    audacious_fileinfopopup_progress_stop(filepopup_win);
+    gchar *last_artwork;
+    fileinfopopup_progress_stop(filepopup_win);
 
-    gchar *last_artwork =
+    last_artwork =
         g_object_get_data(G_OBJECT(filepopup_win), "last_artwork");
     if (last_artwork != NULL)
         g_free(last_artwork);
@@ -308,7 +313,7 @@ audacious_fileinfopopup_destroy(GtkWidget *filepopup_win)
 }
 
 static void
-audacious_fileinfopupup_update_data(GtkWidget *filepopup_win,
+fileinfopupup_update_data(GtkWidget *filepopup_win,
                                     const gchar *text,
                                     const gchar *label_data,
                                     const gchar *header_data)
@@ -327,15 +332,18 @@ audacious_fileinfopupup_update_data(GtkWidget *filepopup_win,
 }
 
 void
-audacious_fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
-                                        TitleInput *tuple)
+fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
+                                        Tuple *tuple)
 {
     gchar *tmp = NULL;
     gint x, y, x_off = 3, y_off = 3, h, w;
-
-    gchar *last_artwork =
-        g_object_get_data(G_OBJECT(filepopup_win), "last_artwork");
+    gchar *length_string, *year_string, *track_string;
+    gchar *last_artwork;
     const static gchar default_artwork[] = DATA_DIR "/images/audio.png";
+    gint length;
+
+    last_artwork =
+        g_object_get_data(G_OBJECT(filepopup_win), "last_artwork");
 
     g_return_if_fail(tuple != NULL);
 
@@ -345,64 +353,63 @@ audacious_fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
         tmp = NULL;
         g_object_set_data(G_OBJECT(filepopup_win), "file", NULL);
     }
-    if (tuple->file_path && tuple->file_name)
+    if (tuple_get_string(tuple, FIELD_FILE_PATH, NULL) && tuple_get_string(tuple, FIELD_FILE_NAME, NULL))
         g_object_set_data(G_OBJECT(filepopup_win), "file",
-                          g_build_filename(tuple->file_path, tuple->file_name,
+                          g_build_filename(tuple_get_string(tuple, FIELD_FILE_PATH, NULL), 
+                                           tuple_get_string(tuple, FIELD_FILE_NAME, NULL),
                                            NULL));
 
     gtk_widget_realize(filepopup_win);
 
-    if (tuple->track_name != NULL)
+    if (tuple_get_string(tuple, FIELD_TITLE, NULL))
     {
         gchar *markup =
             g_markup_printf_escaped("<span style=\"italic\">%s</span>", _("Title"));
         gtk_label_set_markup(GTK_LABEL(g_object_get_data(G_OBJECT(filepopup_win), "header_title")), markup);
         g_free(markup);
-        filepopup_entry_set_text(filepopup_win, "label_title", tuple->track_name);
+        filepopup_entry_set_text(filepopup_win, "label_title", tuple_get_string(tuple, FIELD_TITLE, NULL));
     }
     else
     {
         /* display filename if track_name is not available */
         gchar *markup =
             g_markup_printf_escaped("<span style=\"italic\">%s</span>", _("Filename"));
-        gchar *utf_filename = filename_to_utf8(tuple->file_name);
         gtk_label_set_markup(GTK_LABEL(g_object_get_data(G_OBJECT(filepopup_win), "header_title")), markup);
         g_free(markup);
-        filepopup_entry_set_text(filepopup_win, "label_title", utf_filename);
-        g_free(utf_filename);
+        filepopup_entry_set_text(filepopup_win, "label_title", tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
     }
 
-    audacious_fileinfopupup_update_data(filepopup_win, tuple->performer,
+    fileinfopupup_update_data(filepopup_win, tuple_get_string(tuple, FIELD_ARTIST, NULL),
                                         "label_artist", "header_artist");
-    audacious_fileinfopupup_update_data(filepopup_win, tuple->album_name,
+    fileinfopupup_update_data(filepopup_win, tuple_get_string(tuple, FIELD_ALBUM, NULL),
                                         "label_album", "header_album");
-    audacious_fileinfopupup_update_data(filepopup_win, tuple->genre,
+    fileinfopupup_update_data(filepopup_win, tuple_get_string(tuple, FIELD_GENRE, NULL),
                                         "label_genre", "header_genre");
 
-    gchar *length_string = (tuple->length > 0) ?
-        g_strdup_printf("%d:%02d", tuple->length / 60000, (tuple->length / 1000) % 60) : NULL;
-    audacious_fileinfopupup_update_data(filepopup_win, length_string,
+    length = tuple_get_int(tuple, FIELD_LENGTH, NULL);
+    length_string = (length > 0) ?
+        g_strdup_printf("%d:%02d", length / 60000, (length / 1000) % 60) : NULL;
+    fileinfopupup_update_data(filepopup_win, length_string,
                                         "label_tracklen", "header_tracklen");
     g_free(length_string);
 
-    if ( tuple->length > 0 )
-      g_object_set_data( G_OBJECT(filepopup_win), "length" , GINT_TO_POINTER(tuple->length) );
+    if ( length > 0 )
+      g_object_set_data( G_OBJECT(filepopup_win), "length" , GINT_TO_POINTER(length) );
     else
       g_object_set_data( G_OBJECT(filepopup_win), "length" , GINT_TO_POINTER(-1) );
 
-    gchar *year_string = (tuple->year == 0) ? NULL : g_strdup_printf("%d", tuple->year);
-    audacious_fileinfopupup_update_data(filepopup_win, year_string,
+    year_string = (tuple_get_int(tuple, FIELD_YEAR, NULL) == 0) ? NULL : g_strdup_printf("%d", tuple_get_int(tuple, FIELD_YEAR, NULL));
+    fileinfopupup_update_data(filepopup_win, year_string,
                                         "label_year", "header_year");
     g_free(year_string);
 
-    gchar *track_string = (tuple->track_number == 0) ? NULL : g_strdup_printf("%d", tuple->track_number);
-    audacious_fileinfopupup_update_data(filepopup_win, track_string,
+    track_string = (tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL) == 0) ? NULL : g_strdup_printf("%d", tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL));
+    fileinfopupup_update_data(filepopup_win, track_string,
                                         "label_tracknum", "header_tracknum");
     g_free(track_string);
     
-    if (tuple->file_path && tuple->file_name)
-    {
-        tmp = fileinfo_recursive_get_image(tuple->file_path, tuple->file_name, 0);
+    if (tuple_get_string(tuple, FIELD_FILE_NAME, NULL) && tuple_get_string(tuple, FIELD_FILE_PATH, NULL)) {
+        tmp = fileinfo_recursive_get_image(tuple_get_string(tuple, FIELD_FILE_PATH, NULL), tuple_get_string(tuple, FIELD_FILE_NAME, NULL), 0);
         if (tmp) { // picture found
             if (!last_artwork || strcmp(last_artwork, tmp)) { // new picture
                 filepopup_entry_set_image(filepopup_win, "image_artwork", tmp);
@@ -427,11 +434,11 @@ audacious_fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
 
     /* start a timer that updates a progress bar if the tooltip
        is shown for the song that is being currently played */
-    if (audacious_fileinfopopup_progress_check_active(filepopup_win) == FALSE)
+    if (fileinfopopup_progress_check_active(filepopup_win) == FALSE)
     { 
-        audacious_fileinfopopup_progress_start(filepopup_win);
+        fileinfopopup_progress_start(filepopup_win);
         /* immediately run the callback once to update progressbar status */
-        audacious_fileinfopopup_progress_cb(filepopup_win);
+        fileinfopopup_progress_cb(filepopup_win);
     }
 
     gdk_window_get_pointer(gdk_get_default_root_window(), &x, &y, NULL);
@@ -444,21 +451,21 @@ audacious_fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
 }
 
 void
-audacious_fileinfopopup_show_from_title(GtkWidget *filepopup_win, gchar *title)
+fileinfopopup_show_from_title(GtkWidget *filepopup_win, gchar *title)
 {
-    TitleInput * tuple = bmp_title_input_new();
-    tuple->track_name = g_strdup(title);
-    audacious_fileinfopopup_show_from_tuple(filepopup_win, tuple);
-    bmp_title_input_free(tuple);
+    Tuple * tuple = tuple_new();
+    tuple_associate_string(tuple, FIELD_TITLE, NULL, title);
+    fileinfopopup_show_from_tuple(filepopup_win, tuple);
+    mowgli_object_unref(tuple);
     return;
 }
 
 void
-audacious_fileinfopopup_hide(GtkWidget *filepopup_win, gpointer unused)
+fileinfopopup_hide(GtkWidget *filepopup_win, gpointer unused)
 {
     if (GTK_WIDGET_VISIBLE(filepopup_win))
     {
-        audacious_fileinfopopup_progress_stop(filepopup_win);
+        fileinfopopup_progress_stop(filepopup_win);
 
         gtk_widget_hide(filepopup_win);
 
@@ -466,9 +473,9 @@ audacious_fileinfopopup_hide(GtkWidget *filepopup_win, gpointer unused)
         filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_artist", "");
         filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_album", "");
         filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_genre", "");
-        filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_track", "");
+        filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_tracknum", "");
         filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_year", "");
-        filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_length", "");
+        filepopup_entry_set_text(GTK_WIDGET(filepopup_win), "label_tracklen", "");
 
         gtk_window_resize(GTK_WINDOW(filepopup_win), 1, 1);
     }
