@@ -23,6 +23,8 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+/* #define AUD_DEBUG 1 */
+
 #include "ui_playlist.h"
 
 #include <glib.h>
@@ -68,11 +70,9 @@ GtkWidget *playlistwin;
 
 static GMutex *resize_mutex = NULL;
 
-GtkWidget *playlistwin_list = NULL;
+static GtkWidget *playlistwin_list = NULL;
 GtkWidget *playlistwin_shade, *playlistwin_close;
 
-static GdkBitmap *playlistwin_mask = NULL;
-
 static gboolean playlistwin_hint_flag = FALSE;
 
 static GtkWidget *playlistwin_slider = NULL;
@@ -108,11 +108,9 @@ playlistwin_get_width(void)
 gint
 playlistwin_get_height_unshaded(void)
 {
-    gint height;
     cfg.playlist_height /= PLAYLISTWIN_HEIGHT_SNAP;
     cfg.playlist_height *= PLAYLISTWIN_HEIGHT_SNAP;
-    height = cfg.playlist_height;
-    return height;
+    return cfg.playlist_height;
 }
 
 gint
@@ -130,16 +128,6 @@ playlistwin_get_height(void)
         return playlistwin_get_height_unshaded();
 }
 
-void
-playlistwin_get_size(gint * width, gint * height)
-{
-    if (width)
-        *width = playlistwin_get_width();
-
-    if (height)
-        *height = playlistwin_get_height();
-}
-
 static void
 playlistwin_update_info(Playlist *playlist)
 {
@@ -225,6 +213,8 @@ playlistwin_update_sinfo(Playlist *playlist)
 gboolean
 playlistwin_item_visible(gint index)
 {
+    g_return_val_if_fail(UI_SKINNED_IS_PLAYLIST(playlistwin_list), FALSE);
+
     if (index >= UI_SKINNED_PLAYLIST(playlistwin_list)->first &&
         index < (UI_SKINNED_PLAYLIST(playlistwin_list)->first + UI_SKINNED_PLAYLIST(playlistwin_list)->num_visible) ) {
         return TRUE;
@@ -235,31 +225,31 @@ playlistwin_item_visible(gint index)
 gint
 playlistwin_list_get_visible_count(void)
 {
-    if (playlistwin_list)
+    g_return_val_if_fail(UI_SKINNED_IS_PLAYLIST(playlistwin_list), -1);
+
     return UI_SKINNED_PLAYLIST(playlistwin_list)->num_visible;
-    return (-1);
 }
 
 gint
 playlistwin_list_get_first(void)
 {
-    if (playlistwin_list)
+    g_return_val_if_fail(UI_SKINNED_IS_PLAYLIST(playlistwin_list), -1);
+
     return UI_SKINNED_PLAYLIST(playlistwin_list)->first;
-    return (-1);
 }
 
 gint
 playlistwin_get_toprow(void)
 {
-    if (playlistwin_list)
-        return (UI_SKINNED_PLAYLIST(playlistwin_list)->first);
-    return (-1);
+    g_return_val_if_fail(UI_SKINNED_IS_PLAYLIST(playlistwin_list), -1);
+
+    return (UI_SKINNED_PLAYLIST(playlistwin_list)->first);
 }
 
 void
 playlistwin_set_toprow(gint toprow)
 {
-    if (playlistwin_list)
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list))
         UI_SKINNED_PLAYLIST(playlistwin_list)->first = toprow;
     playlistwin_update_list(playlist_get_active());
 }
@@ -276,28 +266,6 @@ playlistwin_update_list(Playlist *playlist)
     gtk_widget_queue_draw(playlistwin_slider);
 }
 
-static void
-playlistwin_set_mask(void)
-{
-    GdkGC *gc;
-    GdkColor pattern;
-
-    if (playlistwin_mask)
-        g_object_unref(playlistwin_mask);
-
-    playlistwin_mask =
-        gdk_pixmap_new(playlistwin->window, playlistwin_get_width(),
-                       playlistwin_get_height(), 1);
-    gc = gdk_gc_new(playlistwin_mask);
-    pattern.pixel = 1;
-    gdk_gc_set_foreground(gc, &pattern);
-    gdk_draw_rectangle(playlistwin_mask, gc, TRUE, 0, 0,
-                       playlistwin_get_width(), playlistwin_get_height());
-    g_object_unref(gc);
-
-    gtk_widget_shape_combine_mask(playlistwin, playlistwin_mask, 0, 0);
-}
-
 static void
 playlistwin_set_geometry_hints(gboolean shaded)
 {
@@ -332,6 +300,7 @@ playlistwin_set_sinfo_font(gchar *font)
     gchar *tmp = NULL, *tmp2 = NULL;
 
     g_return_if_fail(font);
+    AUDDBG("Attempt to set font \"%s\"\n", font);
 
     tmp = g_strdup(font);
     g_return_if_fail(tmp);
@@ -340,7 +309,7 @@ playlistwin_set_sinfo_font(gchar *font)
     tmp2 = g_strdup_printf("%s 8", tmp);
     g_return_if_fail(tmp2);
 
-    ui_skinned_textbox_set_xfont(playlistwin_sinfo, cfg.mainwin_use_xfont, tmp2);
+    ui_skinned_textbox_set_xfont(playlistwin_sinfo, !cfg.mainwin_use_bitmapfont, tmp2);
 
     g_free(tmp);
     g_free(tmp2);
@@ -382,8 +351,6 @@ playlistwin_set_shade(gboolean shaded)
     gtk_window_resize(GTK_WINDOW(playlistwin),
                       playlistwin_get_width(),
                       playlistwin_get_height());
-
-    playlistwin_set_mask();
 }
 
 static void
@@ -417,7 +384,8 @@ playlistwin_release(GtkWidget * widget,
 void
 playlistwin_scroll(gint num)
 {
-    UI_SKINNED_PLAYLIST(playlistwin_list)->first += num;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list))
+        UI_SKINNED_PLAYLIST(playlistwin_list)->first += num;
     playlistwin_update_list(playlist_get_active());
 }
 
@@ -439,9 +407,11 @@ playlistwin_select_all(void)
     Playlist *playlist = playlist_get_active();
 
     playlist_select_all(playlist, TRUE);
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = 0;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = 0;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_max = playlist_get_length(playlist) - 1;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list)) {
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = 0;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = 0;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_max = playlist_get_length(playlist) - 1;
+    }
     playlistwin_update_list(playlist);
 }
 
@@ -449,8 +419,10 @@ static void
 playlistwin_select_none(void)
 {
     playlist_select_all(playlist_get_active(), FALSE);
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = -1;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = -1;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list)) {
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = -1;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = -1;
+    }
     playlistwin_update_list(playlist_get_active());
 }
 
@@ -570,14 +542,23 @@ playlistwin_select_search(void)
          /* create a TitleInput tuple with user search data */
          Tuple *tuple = tuple_new();
          gchar *searchdata = NULL;
+
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_title) );
+         AUDDBG("title=\"%s\"\n", searchdata);
          tuple_associate_string(tuple, FIELD_TITLE, NULL, searchdata);
+
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_album) );
+         AUDDBG("album=\"%s\"\n", searchdata);
          tuple_associate_string(tuple, FIELD_ALBUM, NULL, searchdata);
+
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_performer) );
+         AUDDBG("performer=\"%s\"\n", searchdata);
          tuple_associate_string(tuple, FIELD_ARTIST, NULL, searchdata);
+
          searchdata = (gchar*)gtk_entry_get_text( GTK_ENTRY(searchdlg_entry_file_name) );
+         AUDDBG("filename=\"%s\"\n", searchdata);
          tuple_associate_string(tuple, FIELD_FILE_NAME, NULL, searchdata);
+
          /* check if previous selection should be cleared before searching */
          if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(searchdlg_checkbt_clearprevsel)) == TRUE )
              playlistwin_select_none();
@@ -605,8 +586,10 @@ static void
 playlistwin_inverse_selection(void)
 {
     playlist_select_invert_all(playlist_get_active());
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = -1;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = -1;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list)) {
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_selected = -1;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->prev_min = -1;
+    }
     playlistwin_update_list(playlist_get_active());
 }
 
@@ -665,8 +648,6 @@ playlistwin_resize(gint width, gint height)
     ui_skinned_button_move_relative(playlistwin_sscroll_up, dx, dy);
     ui_skinned_button_move_relative(playlistwin_sscroll_down, dx, dy);
 
-    playlistwin_set_mask();
-
     gtk_widget_set_size_request(playlistwin_sinfo, playlistwin_get_width() - 35,
                                 bmp_active_skin->properties.textbox_bitmap_font_height);
     GList *iter;
@@ -696,13 +677,13 @@ playlistwin_motion(GtkWidget * widget,
         {
             playlistwin_resize(event->x + playlistwin_resize_x,
                                event->y + playlistwin_resize_y);
+            gdk_window_resize(playlistwin->window,
+                              cfg.playlist_width, playlistwin_get_height());
+            gdk_flush();
         }
-        gdk_window_resize(playlistwin->window,
-                          cfg.playlist_width, cfg.playlist_height);
     }
     else if (dock_is_moving(GTK_WINDOW(playlistwin)))
         dock_move_motion(GTK_WINDOW(playlistwin), event);
-    gdk_flush();
 
     while ((gevent = gdk_event_get()) != NULL) gdk_event_free(gevent);
 }
@@ -1072,7 +1053,6 @@ playlistwin_press(GtkWidget * widget,
     {
         dock_move_press(dock_window_list, GTK_WINDOW(playlistwin), event,
                         FALSE);
-        gtk_window_present(GTK_WINDOW(playlistwin));
     }
     else if (event->button == 1 && event->type == GDK_2BUTTON_PRESS
              && event->y < 14) {
@@ -1170,10 +1150,12 @@ playlistwin_keypress_up_down_handler(UiSkinnedPlaylist * pl,
 static gboolean
 playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 {
+    g_return_val_if_fail(UI_SKINNED_IS_PLAYLIST(playlistwin_list), FALSE);
     Playlist *playlist = playlist_get_active();
 
     guint keyval;
     gboolean refresh = FALSE;
+    guint cur_pos;
 
     if (cfg.playlist_shaded)
         return FALSE;
@@ -1264,6 +1246,12 @@ playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
                 gtk_window_present(GTK_WINDOW(equalizerwin));
         }
         break;
+    case GDK_space:
+        cur_pos=playlist_get_position(playlist);
+        UI_SKINNED_PLAYLIST(playlistwin_list)->first =
+            cur_pos - (UI_SKINNED_PLAYLIST(playlistwin_list)->num_visible >> 1);
+        refresh = TRUE;
+        break;
     default:
         return FALSE;
     }
@@ -1319,9 +1307,11 @@ playlistwin_drag_motion(GtkWidget * widget,
                         GtkSelectionData * selection_data,
                         guint info, guint time, gpointer user_data)
 {
-    UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion = TRUE;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion_x = x;
-    UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion_y = y;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list)) {
+        UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion = TRUE;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion_x = x;
+        UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion_y = y;
+    }
     playlistwin_update_list(playlist_get_active());
     playlistwin_hint_flag = TRUE;
 }
@@ -1330,7 +1320,8 @@ static void
 playlistwin_drag_end(GtkWidget * widget,
                      GdkDragContext * context, gpointer user_data)
 {
-    UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion = FALSE;
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list))
+        UI_SKINNED_PLAYLIST(playlistwin_list)->drag_motion = FALSE;
     playlistwin_hint_flag = FALSE;
     playlistwin_update_list(playlist_get_active());
 }
@@ -1352,7 +1343,8 @@ playlistwin_drag_data_received(GtkWidget * widget,
         g_message("Received no DND data!");
         return;
     }
-    if (x < playlistwin_get_width() - 20 || y < cfg.playlist_height - 38) {
+    if (UI_SKINNED_IS_PLAYLIST(playlistwin_list) &&
+        (x < playlistwin_get_width() - 20 || y < cfg.playlist_height - 38)) {
         pos = y / UI_SKINNED_PLAYLIST(playlistwin_list)->fheight + UI_SKINNED_PLAYLIST(playlistwin_list)->first;
 
         pos = MIN(pos, playlist_get_length(playlist));
@@ -1522,8 +1514,6 @@ playlistwin_create_window(void)
     gtk_window_set_icon(GTK_WINDOW(playlistwin), icon);
     g_object_unref(icon);
 
-    gtk_widget_set_app_paintable(playlistwin, TRUE);
-
     if (cfg.playlist_x != -1 && cfg.save_window_position)
         gtk_window_move(GTK_WINDOW(playlistwin),
                         cfg.playlist_x, cfg.playlist_y);
@@ -1565,8 +1555,6 @@ playlistwin_create_window(void)
                      G_CALLBACK(playlistwin_keypress), NULL);
     g_signal_connect(playlistwin, "selection_received",
                      G_CALLBACK(selection_received), NULL);
-
-    playlistwin_set_mask();
 }
 
 void
@@ -1931,8 +1919,13 @@ playlistwin_select_search_kp_cb(GtkWidget *entry, GdkEventKey *event,
     switch (event->keyval)
     {
         case GDK_Return:
-            gtk_dialog_response(GTK_DIALOG(searchdlg_win), GTK_RESPONSE_ACCEPT);
-            return TRUE;
+            if (gtk_im_context_filter_keypress (GTK_ENTRY (entry)->im_context, event)) {
+                GTK_ENTRY (entry)->need_im_reset = TRUE;
+                return TRUE;
+            } else {
+                gtk_dialog_response(GTK_DIALOG(searchdlg_win), GTK_RESPONSE_ACCEPT);
+                return TRUE;
+            }
         default:
             return FALSE;
     }
