@@ -47,19 +47,19 @@ enum {
 
 struct _UiSkinnedTextboxPrivate {
     SkinPixmapId     skin_index;
-    gboolean         double_size;
+    gboolean         scaled;
     gboolean         scroll_back;
     gint             nominal_y, nominal_height;
     gint             scroll_timeout;
     gint             font_ascent, font_descent;
     PangoFontDescription *font;
     gchar            *fontname;
-    gchar            *pixmap_text;
+    gchar            *pixbuf_text;
     gint             skin_id;
     gint             drag_x, drag_off, offset;
     gboolean         is_scrollable, is_dragging;
-    gint             pixmap_width;
-    GdkPixmap        *pixmap;
+    gint             pixbuf_width;
+    GdkPixbuf        *pixbuf;
     gboolean         scroll_allowed, scroll_enabled;
     gint             scroll_dummy;
     gint             move_x, move_y;
@@ -75,7 +75,7 @@ static gboolean ui_skinned_textbox_expose         (GtkWidget *widget, GdkEventEx
 static gboolean ui_skinned_textbox_button_press   (GtkWidget *widget, GdkEventButton *event);
 static gboolean ui_skinned_textbox_button_release (GtkWidget *widget, GdkEventButton *event);
 static gboolean ui_skinned_textbox_motion_notify  (GtkWidget *widget, GdkEventMotion *event);
-static void ui_skinned_textbox_toggle_doublesize  (UiSkinnedTextbox *textbox);
+static void ui_skinned_textbox_toggle_scaled      (UiSkinnedTextbox *textbox);
 static void ui_skinned_textbox_redraw             (UiSkinnedTextbox *textbox);
 static gboolean ui_skinned_textbox_should_scroll  (UiSkinnedTextbox *textbox);
 static void textbox_generate_xfont_pixmap         (UiSkinnedTextbox *textbox, const gchar *pixmaptext);
@@ -129,7 +129,7 @@ static void ui_skinned_textbox_class_init(UiSkinnedTextboxClass *klass) {
     klass->clicked = NULL;
     klass->double_clicked = NULL;
     klass->right_clicked = NULL;
-    klass->doubled = ui_skinned_textbox_toggle_doublesize;
+    klass->scaled = ui_skinned_textbox_toggle_scaled;
     klass->redraw = ui_skinned_textbox_redraw;
 
     textbox_signals[CLICKED] = 
@@ -145,11 +145,11 @@ static void ui_skinned_textbox_class_init(UiSkinnedTextboxClass *klass) {
     textbox_signals[RIGHT_CLICKED] = 
         g_signal_new ("right-clicked", G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                       G_STRUCT_OFFSET (UiSkinnedTextboxClass, right_clicked), NULL, NULL,
-                      gtk_marshal_VOID__VOID, G_TYPE_NONE, 0);
+                      gtk_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
 
     textbox_signals[DOUBLED] = 
-        g_signal_new ("toggle-double-size", G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
-                      G_STRUCT_OFFSET (UiSkinnedTextboxClass, doubled), NULL, NULL,
+        g_signal_new ("toggle-scaled", G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+                      G_STRUCT_OFFSET (UiSkinnedTextboxClass, scaled), NULL, NULL,
                       gtk_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     textbox_signals[REDRAW] = 
@@ -183,7 +183,7 @@ GtkWidget* ui_skinned_textbox_new(GtkWidget *fixed, gint x, gint y, gint w, gboo
     priv->scroll_timeout = 0;
     priv->scroll_dummy = 0;
 
-    priv->double_size = FALSE;
+    priv->scaled = FALSE;
 
     gtk_fixed_put(GTK_FIXED(fixed), GTK_WIDGET(textbox), textbox->x, textbox->y);
 
@@ -238,8 +238,8 @@ static void ui_skinned_textbox_size_request(GtkWidget *widget, GtkRequisition *r
     UiSkinnedTextbox *textbox = UI_SKINNED_TEXTBOX(widget);
     UiSkinnedTextboxPrivate *priv = UI_SKINNED_TEXTBOX_GET_PRIVATE(textbox);
 
-    requisition->width = textbox->width*(1+priv->double_size);
-    requisition->height = textbox->height*(1+priv->double_size);
+    requisition->width = textbox->width*(priv->scaled ? cfg.scale_factor : 1);
+    requisition->height = textbox->height*(priv->scaled ?  cfg.scale_factor : 1 );
 }
 
 static void ui_skinned_textbox_size_allocate(GtkWidget *widget, GtkAllocation *allocation) {
@@ -247,22 +247,22 @@ static void ui_skinned_textbox_size_allocate(GtkWidget *widget, GtkAllocation *a
     UiSkinnedTextboxPrivate *priv = UI_SKINNED_TEXTBOX_GET_PRIVATE(textbox);
 
     widget->allocation = *allocation;
-    widget->allocation.x *= (1+priv->double_size);
-    widget->allocation.y *= (1+priv->double_size);
+    widget->allocation.x *= (priv->scaled ? cfg.scale_factor : 1);
+    widget->allocation.y *= (priv->scaled ? cfg.scale_factor : 1);
     if (GTK_WIDGET_REALIZED (widget))
         gdk_window_move_resize(widget->window, widget->allocation.x, widget->allocation.y, allocation->width, allocation->height);
 
-    if (textbox->x + priv->move_x == widget->allocation.x/(priv->double_size ? 2 : 1))
+    if (textbox->x + priv->move_x - widget->allocation.x/(priv->scaled ? cfg.scale_factor : 1) <3);
         priv->move_x = 0;
-    if (textbox->y + priv->move_y == widget->allocation.y/(priv->double_size ? 2 : 1))
+    if (textbox->y + priv->move_y - widget->allocation.y/(priv->scaled ? cfg.scale_factor : 1) <3);
         priv->move_y = 0;
-    textbox->x = widget->allocation.x/(priv->double_size ? 2 : 1);
-    textbox->y = widget->allocation.y/(priv->double_size ? 2 : 1);
+    textbox->x = widget->allocation.x/(priv->scaled ? cfg.scale_factor : 1);
+    textbox->y = widget->allocation.y/(priv->scaled ? cfg.scale_factor : 1);
 
-    if (textbox->width != widget->allocation.width/(priv->double_size ? 2 : 1)) {
-            textbox->width = widget->allocation.width/(priv->double_size ? 2 : 1);
-            if (priv->pixmap_text) g_free(priv->pixmap_text);
-            priv->pixmap_text = NULL;
+    if (textbox->width - (guint) (widget->allocation.width / (priv->scaled ? cfg.scale_factor : 1)) > 2) {
+            textbox->width = (guint) (widget->allocation.width / (priv->scaled ? cfg.scale_factor : 1));
+            if (priv->pixbuf_text) g_free(priv->pixbuf_text);
+            priv->pixbuf_text = NULL;
             priv->offset = 0;
             gtk_widget_set_size_request(widget, textbox->width, textbox->height);
             gtk_widget_queue_draw(GTK_WIDGET(textbox));
@@ -278,65 +278,48 @@ static gboolean ui_skinned_textbox_expose(GtkWidget *widget, GdkEventExpose *eve
     UiSkinnedTextboxPrivate *priv = UI_SKINNED_TEXTBOX_GET_PRIVATE(textbox);
     g_return_val_if_fail (textbox->width > 0 && textbox->height > 0, FALSE);
 
-    GdkPixmap *obj = NULL;
-    GdkGC *gc;
+    GdkPixbuf *obj = NULL;
     gint cw;
 
-    if (textbox->text && (!priv->pixmap_text || strcmp(textbox->text, priv->pixmap_text)))
+    if (textbox->text && (!priv->pixbuf_text || strcmp(textbox->text, priv->pixbuf_text)))
         textbox_generate_pixmap(textbox);
 
-    if (priv->pixmap) {
+    if (priv->pixbuf) {
         if (skin_get_id() != priv->skin_id) {
             priv->skin_id = skin_get_id();
             textbox_generate_pixmap(textbox);
         }
-        obj = gdk_pixmap_new(NULL, textbox->width, textbox->height, gdk_rgb_get_visual()->depth);
-        gc = gdk_gc_new(obj);
+        obj = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, textbox->width, textbox->height);
 
         if (cfg.twoway_scroll) { // twoway scroll
-            cw = priv->pixmap_width - priv->offset;
+            cw = priv->pixbuf_width - priv->offset;
             if (cw > textbox->width)
                 cw = textbox->width;
-            gdk_draw_drawable(obj, gc, priv->pixmap, priv->offset, 0, 0, 0, cw, textbox->height);
+            gdk_pixbuf_copy_area(priv->pixbuf, priv->offset, 0, cw, textbox->height, obj, 0, 0);
             if (cw < textbox->width)
-                gdk_draw_drawable(obj, gc, priv->pixmap, 0, 0,
-                                  textbox->x + cw, textbox->y,
-                                  textbox->width - cw, textbox->height);
+                gdk_pixbuf_copy_area(priv->pixbuf, 0, 0, textbox->width - cw, textbox->height,
+                                     obj, textbox->width - cw, textbox->height);
         } else { // oneway scroll
             int cw1, cw2;
 
-            if (priv->offset >= priv->pixmap_width)
+            if (priv->offset >= priv->pixbuf_width)
                 priv->offset = 0;
 
-            if (priv->pixmap_width - priv->offset > textbox->width) { // case1
+            if (priv->pixbuf_width - priv->offset > textbox->width) { // case1
                 cw1 = textbox->width;
-                gdk_draw_drawable(obj, gc, priv->pixmap, priv->offset, 0,
-                                  0, 0, cw1, textbox->height);
+                gdk_pixbuf_copy_area(priv->pixbuf, priv->offset, 0, cw1, textbox->height,
+                                     obj, 0, 0);
             } else { // case 2
-                cw1 = priv->pixmap_width - priv->offset;
-                gdk_draw_drawable(obj, gc, priv->pixmap, priv->offset, 0,
-                                  0, 0, cw1, textbox->height);
+                cw1 = priv->pixbuf_width - priv->offset;
+                gdk_pixbuf_copy_area(priv->pixbuf, priv->offset, 0, cw1, textbox->height, obj, 0, 0);
                 cw2 = textbox->width - cw1;
-                gdk_draw_drawable(obj, gc, priv->pixmap, 0, 0, cw1, 0, cw2, textbox->height);
+                gdk_pixbuf_copy_area(priv->pixbuf, 0, 0, cw2, textbox->height, obj, cw1, 0);
             }
-
         }
 
-        GdkPixmap *image = NULL;
-
-        if (priv->double_size) {
-            image = create_dblsize_pixmap(obj);
-        } else {
-            image = gdk_pixmap_new(NULL, textbox->width, textbox->height, gdk_rgb_get_visual()->depth);
-            gdk_draw_drawable (image, gc, obj, 0, 0, 0, 0, textbox->width, textbox->height);
-        }
+        ui_skinned_widget_draw(widget, obj, textbox->width, textbox->height, priv->scaled);
 
         g_object_unref(obj);
-
-        gdk_draw_drawable (widget->window, gc, image, 0, 0, 0, 0,
-                           textbox->width*(1+priv->double_size), textbox->height*(1+priv->double_size));
-        g_object_unref(gc);
-        g_object_unref(image);
     }
 
     return FALSE;
@@ -356,7 +339,7 @@ static gboolean ui_skinned_textbox_button_press(GtkWidget *widget, GdkEventButto
             return FALSE;
         else if (event->button == 1) {
             if (priv->scroll_allowed) {
-                if ((priv->pixmap_width > textbox->width) && priv->is_scrollable) {
+                if ((priv->pixbuf_width > textbox->width) && priv->is_scrollable) {
                     priv->is_dragging = TRUE;
                     priv->drag_off = priv->offset;
                     priv->drag_x = event->x;
@@ -365,7 +348,7 @@ static gboolean ui_skinned_textbox_button_press(GtkWidget *widget, GdkEventButto
                 g_signal_emit(widget, textbox_signals[CLICKED], 0);
 
         } else if (event->button == 3) {
-            g_signal_emit(widget, textbox_signals[RIGHT_CLICKED], 0);
+            g_signal_emit(widget, textbox_signals[RIGHT_CLICKED], 0, event);
         } else
             priv->is_dragging = FALSE;
     } else if (event->type == GDK_2BUTTON_PRESS) {
@@ -399,14 +382,14 @@ static gboolean ui_skinned_textbox_motion_notify(GtkWidget *widget, GdkEventMoti
 
     if (priv->is_dragging) {
         if (priv->scroll_allowed &&
-            priv->pixmap_width > textbox->width) {
+            priv->pixbuf_width > textbox->width) {
             priv->offset = priv->drag_off - (event->x - priv->drag_x);
 
             while (priv->offset < 0)
                 priv->offset = 0;
 
-            while (priv->offset > (priv->pixmap_width - textbox->width))
-                priv->offset = (priv->pixmap_width - textbox->width);
+            while (priv->offset > (priv->pixbuf_width - textbox->width))
+                priv->offset = (priv->pixbuf_width - textbox->width);
 
             gtk_widget_queue_draw(widget);
         }
@@ -415,13 +398,14 @@ static gboolean ui_skinned_textbox_motion_notify(GtkWidget *widget, GdkEventMoti
   return TRUE;
 }
 
-static void ui_skinned_textbox_toggle_doublesize(UiSkinnedTextbox *textbox) {
+static void ui_skinned_textbox_toggle_scaled(UiSkinnedTextbox *textbox) {
     GtkWidget *widget = GTK_WIDGET (textbox);
     UiSkinnedTextboxPrivate *priv = UI_SKINNED_TEXTBOX_GET_PRIVATE(textbox);
 
-    priv->double_size = !priv->double_size;
+    priv->scaled = !priv->scaled;
 
-    gtk_widget_set_size_request(widget, textbox->width*(1+priv->double_size), textbox->height*(1+priv->double_size));
+    gtk_widget_set_size_request(widget, textbox->width*(priv->scaled ? cfg.scale_factor : 1 ), 
+    textbox->height*(priv->scaled ? cfg.scale_factor : 1 ));
 
     gtk_widget_queue_draw(GTK_WIDGET(textbox));
 }
@@ -475,9 +459,9 @@ void ui_skinned_textbox_set_xfont(GtkWidget *widget, gboolean use_xfont, const g
     textbox->height = priv->nominal_height;
 
     /* Make sure the pixmap is regenerated */
-    if (priv->pixmap_text) {
-        g_free(priv->pixmap_text);
-        priv->pixmap_text = NULL;
+    if (priv->pixbuf_text) {
+        g_free(priv->pixbuf_text);
+        priv->pixbuf_text = NULL;
     }
 
     if (!use_xfont || strlen(fontname) == 0)
@@ -519,12 +503,14 @@ void ui_skinned_textbox_set_text(GtkWidget *widget, const gchar *text) {
 }
 
 static void textbox_generate_xfont_pixmap(UiSkinnedTextbox *textbox, const gchar *pixmaptext) {
+    /* FIXME: should operate directly on priv->pixbuf, it shouldn't use pixmap */
     gint length, i;
     GdkGC *gc, *maskgc;
     GdkColor *c, pattern;
     GdkBitmap *mask;
     PangoLayout *layout;
     gint width;
+    GdkPixmap *pixmap;
 
     g_return_if_fail(textbox != NULL);
     g_return_if_fail(pixmaptext != NULL);
@@ -536,23 +522,23 @@ static void textbox_generate_xfont_pixmap(UiSkinnedTextbox *textbox, const gchar
 
     text_get_extents(priv->fontname, pixmaptext, &width, NULL, NULL, NULL);
 
-    priv->pixmap_width = MAX(width, textbox->width);
-    priv->pixmap = gdk_pixmap_new(mainwin->window, priv->pixmap_width,
+    priv->pixbuf_width = MAX(width, textbox->width);
+    pixmap = gdk_pixmap_new(mainwin->window, priv->pixbuf_width,
                                    textbox->height,
                                    gdk_rgb_get_visual()->depth);
-    gc = gdk_gc_new(priv->pixmap);
+    gc = gdk_gc_new(pixmap);
     c = skin_get_color(bmp_active_skin, SKIN_TEXTBG);
     for (i = 0; i < textbox->height; i++) {
         gdk_gc_set_foreground(gc, &c[6 * i / textbox->height]);
-        gdk_draw_line(priv->pixmap, gc, 0, i, priv->pixmap_width, i);
+        gdk_draw_line(pixmap, gc, 0, i, priv->pixbuf_width, i);
     }
 
-    mask = gdk_pixmap_new(mainwin->window, priv->pixmap_width, textbox->height, 1);
+    mask = gdk_pixmap_new(mainwin->window, priv->pixbuf_width, textbox->height, 1);
     maskgc = gdk_gc_new(mask);
     pattern.pixel = 0;
     gdk_gc_set_foreground(maskgc, &pattern);
 
-    gdk_draw_rectangle(mask, maskgc, TRUE, 0, 0, priv->pixmap_width, textbox->height);
+    gdk_draw_rectangle(mask, maskgc, TRUE, 0, 0, priv->pixbuf_width, textbox->height);
     pattern.pixel = 1;
     gdk_gc_set_foreground(maskgc, &pattern);
 
@@ -561,7 +547,7 @@ static void textbox_generate_xfont_pixmap(UiSkinnedTextbox *textbox, const gchar
     layout = gtk_widget_create_pango_layout(mainwin, pixmaptext);
     pango_layout_set_font_description(layout, priv->font);
 
-    gdk_draw_layout(priv->pixmap, gc, 0, (priv->font_descent / 2), layout);
+    gdk_draw_layout(pixmap, gc, 0, (priv->font_descent / 2), layout);
     g_object_unref(layout);
 
     g_object_unref(maskgc);
@@ -570,8 +556,9 @@ static void textbox_generate_xfont_pixmap(UiSkinnedTextbox *textbox, const gchar
     c = skin_get_color(bmp_active_skin, SKIN_TEXTFG);
     for (i = 0; i < textbox->height; i++) {
         gdk_gc_set_foreground(gc, &c[6 * i / textbox->height]);
-        gdk_draw_line(priv->pixmap, gc, 0, i, priv->pixmap_width, i);
+        gdk_draw_line(pixmap, gc, 0, i, priv->pixbuf_width, i);
     }
+    priv->pixbuf = gdk_pixbuf_get_from_drawable(NULL, pixmap, gdk_colormap_get_system(), 0, 0, 0, 0, priv->pixbuf_width, textbox->height);
     g_object_unref(mask);
     g_object_unref(gc);
 }
@@ -590,10 +577,10 @@ static gboolean textbox_scroll(gpointer data) {
                 else
                     priv->offset += 1;
 
-                if (priv->offset >= (priv->pixmap_width - textbox->width)) {
+                if (priv->offset >= (priv->pixbuf_width - textbox->width)) {
                     priv->scroll_back = TRUE;
                     priv->scroll_dummy = 0;
-                    priv->offset = priv->pixmap_width - textbox->width;
+                    priv->offset = priv->pixbuf_width - textbox->width;
                 }
                 if (priv->offset <= 0) {
                     priv->scroll_back = FALSE;
@@ -615,14 +602,13 @@ static void textbox_generate_pixmap(UiSkinnedTextbox *textbox) {
     gint length, i, x, y, wl;
     gchar *pixmaptext;
     gchar *tmp, *stxt;
-    GdkGC *gc;
 
     g_return_if_fail(textbox != NULL);
     UiSkinnedTextboxPrivate *priv = UI_SKINNED_TEXTBOX_GET_PRIVATE(textbox);
 
-    if (priv->pixmap) {
-        g_object_unref(priv->pixmap);
-        priv->pixmap = NULL;
+    if (priv->pixbuf) {
+        g_object_unref(priv->pixbuf);
+        priv->pixbuf = NULL;
     }
 
     /*
@@ -630,13 +616,13 @@ static void textbox_generate_pixmap(UiSkinnedTextbox *textbox) {
      * changed.  This is a hack to avoid visual noice on vbr files
      * where we guess the length.
      */
-    if (!(priv->pixmap_text && strrchr(textbox->text, '(') &&
-          !strncmp(priv->pixmap_text, textbox->text,
+    if (!(priv->pixbuf_text && strrchr(textbox->text, '(') &&
+          !strncmp(priv->pixbuf_text, textbox->text,
                    strrchr(textbox->text, '(') - textbox->text)))
         priv->offset = 0;
 
-    g_free(priv->pixmap_text);
-    priv->pixmap_text = g_strdup(textbox->text);
+    g_free(priv->pixbuf_text);
+    priv->pixbuf_text = g_strdup(textbox->text);
 
     /*
      * wl is the number of (partial) letters visible. Only makes
@@ -654,20 +640,20 @@ static void textbox_generate_pixmap(UiSkinnedTextbox *textbox) {
 
     if (priv->is_scrollable) {
         if(!cfg.twoway_scroll) {
-            pixmaptext = g_strdup_printf("%s *** ", priv->pixmap_text);
+            pixmaptext = g_strdup_printf("%s *** ", priv->pixbuf_text);
             length += 5;
         } else
-            pixmaptext = g_strdup(priv->pixmap_text);
+            pixmaptext = g_strdup(priv->pixbuf_text);
     } else
     if (!priv->font && length <= wl) {
         gint pad = wl - length;
         gchar *padchars = g_strnfill(pad, ' ');
 
-        pixmaptext = g_strconcat(priv->pixmap_text, padchars, NULL);
+        pixmaptext = g_strconcat(priv->pixbuf_text, padchars, NULL);
         g_free(padchars);
         length += pad;
     } else
-        pixmaptext = g_strdup(priv->pixmap_text);
+        pixmaptext = g_strdup(priv->pixbuf_text);
 
     if (priv->is_scrollable) {
         if (priv->scroll_enabled && !priv->scroll_timeout) {
@@ -689,11 +675,9 @@ static void textbox_generate_pixmap(UiSkinnedTextbox *textbox) {
         return;
     }
 
-    priv->pixmap_width = length * bmp_active_skin->properties.textbox_bitmap_font_width;
-    priv->pixmap = gdk_pixmap_new(NULL,
-                                     priv->pixmap_width, bmp_active_skin->properties.textbox_bitmap_font_height,
-                                     gdk_rgb_get_visual()->depth);
-    gc = gdk_gc_new(priv->pixmap);
+    priv->pixbuf_width = length * bmp_active_skin->properties.textbox_bitmap_font_width;
+    priv->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
+                                  priv->pixbuf_width, bmp_active_skin->properties.textbox_bitmap_font_height);
 
     for (tmp = stxt = g_utf8_strup(pixmaptext, -1), i = 0;
          tmp != NULL && i < length; i++, tmp = g_utf8_next_char(tmp)) {
@@ -710,15 +694,14 @@ static void textbox_generate_pixmap(UiSkinnedTextbox *textbox) {
         else
             textbox_handle_special_char(tmp, &x, &y);
 
-        skin_draw_pixmap(GTK_WIDGET(textbox), bmp_active_skin,
-                         priv->pixmap, gc, priv->skin_index,
+        skin_draw_pixbuf(GTK_WIDGET(textbox), bmp_active_skin,
+                         priv->pixbuf, priv->skin_index,
                          x, y, i * bmp_active_skin->properties.textbox_bitmap_font_width, 0,
                          bmp_active_skin->properties.textbox_bitmap_font_width, 
                          bmp_active_skin->properties.textbox_bitmap_font_height);
     }
     g_free(stxt);
     g_free(pixmaptext);
-    g_object_unref(gc);
 }
 
 void ui_skinned_textbox_set_scroll(GtkWidget *widget, gboolean scroll) {
